@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -28,6 +28,7 @@ impl Span {
         self.offset != usize::MAX
     }
 
+    #[allow(clippy::needless_update)] // needed when `proc_macro_span` is enabled
     pub fn new(offset: usize) -> Self {
         Self { offset, ..Default::default() }
     }
@@ -257,7 +258,12 @@ impl BuildDiagnostics {
         span: SourceLocation,
         level: DiagnosticLevel,
     ) {
-        self.inner.push(Diagnostic { message, span, level }.into());
+        debug_assert!(
+            !message.as_str().ends_with('.'),
+            "Error message should not end with a period: ({:?})",
+            message
+        );
+        self.inner.push(Diagnostic { message, span, level });
     }
     pub fn push_error_with_span(&mut self, message: String, span: SourceLocation) {
         self.push_diagnostic_with_span(message, span, DiagnosticLevel::Error)
@@ -265,8 +271,14 @@ impl BuildDiagnostics {
     pub fn push_error(&mut self, message: String, source: &dyn Spanned) {
         self.push_error_with_span(message, source.to_source_location());
     }
+    pub fn push_warning_with_span(&mut self, message: String, span: SourceLocation) {
+        self.push_diagnostic_with_span(message, span, DiagnosticLevel::Warning)
+    }
+    pub fn push_warning(&mut self, message: String, source: &dyn Spanned) {
+        self.push_warning_with_span(message, source.to_source_location());
+    }
     pub fn push_compiler_error(&mut self, error: Diagnostic) {
-        self.inner.push(error.into());
+        self.inner.push(error);
     }
 
     pub fn push_property_deprecation_warning(
@@ -296,9 +308,9 @@ impl BuildDiagnostics {
     }
 
     #[cfg(feature = "display-diagnostics")]
-    fn call_diagnostics<'a, Output>(
+    fn call_diagnostics<Output>(
         self,
-        output: &'a mut Output,
+        output: &mut Output,
         mut handle_no_source: Option<&mut dyn FnMut(Diagnostic)>,
         emitter_factory: impl for<'b> FnOnce(
             &'b mut Output,
@@ -388,7 +400,7 @@ impl BuildDiagnostics {
         self.call_diagnostics(
             &mut (),
             Some(&mut |diag| {
-                let span = diag.span.span.span.clone().or_else(|| {
+                let span = diag.span.span.span.or_else(|| {
                     //let pos =
                     //span_map.binary_search_by_key(d.span.offset, |x| x.0).unwrap_or_else(|x| x);
                     //d.span.span = span_map.get(pos).as_ref().map(|x| x.1);
