@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -13,7 +13,7 @@ use crate::layout::Orientation;
 use crate::object_tree::*;
 use crate::parser::{NodeOrToken, SyntaxNode};
 use core::cell::RefCell;
-use std::collections::{BTreeMap, HashMap};
+use std::collections::HashMap;
 use std::rc::{Rc, Weak};
 
 // FIXME remove the pub
@@ -160,7 +160,14 @@ impl BuiltinFunction {
             BuiltinFunction::ShowPopupWindow => false,
             BuiltinFunction::StringToFloat | BuiltinFunction::StringIsFloat => true,
             BuiltinFunction::ColorBrighter | BuiltinFunction::ColorDarker => true,
+            // ImageSize is pure, except when loading images via the network. Then the initial size will be 0/0 and
+            // we need to make sure that calls to this function stay within a binding, so that the property
+            // notification when updating kicks in. Only the online editor (wasm-interpreter) loads images via the network,
+            // which is when this code is targeting wasm.
+            #[cfg(not(target_arch = "wasm32"))]
             BuiltinFunction::ImageSize => true,
+            #[cfg(target_arch = "wasm32")]
+            BuiltinFunction::ImageSize => false,
             BuiltinFunction::Rgb => true,
             BuiltinFunction::ImplicitLayoutInfo(_) => false,
             BuiltinFunction::RegisterCustomFontByPath
@@ -262,11 +269,11 @@ declare_units! {
     /// Degree
     Deg = "deg" -> Angle,
     /// Gradians
-    Grad = "grad" -> Angle * 400./360.,
+    Grad = "grad" -> Angle * 360./180.,
     /// Turns
-    Turn = "turn" -> Angle * 1./360.,
+    Turn = "turn" -> Angle * 360.,
     /// Radians
-    Rad = "rad" -> Angle * std::f32::consts::TAU/360.,
+    Rad = "rad" -> Angle * 360./std::f32::consts::TAU,
 }
 
 impl Default for Unit {
@@ -511,8 +518,12 @@ impl Expression {
                 let false_type = false_expr.ty();
                 if true_type == false_type {
                     true_type
+                } else if true_type == Type::Invalid {
+                    false_type
+                } else if false_type == Type::Invalid {
+                    true_type
                 } else {
-                    Type::Invalid
+                    Type::Void
                 }
             }
             Expression::BinaryExpression { op, lhs, rhs } => {
@@ -529,7 +540,7 @@ impl Expression {
                     debug_assert!(*op == '*' || *op == '/');
                     let unit_vec = |ty| {
                         if let Type::UnitProduct(v) = ty {
-                            v.clone()
+                            v
                         } else if let Some(u) = ty.default_unit() {
                             vec![(u, 1)]
                         } else {
@@ -654,7 +665,7 @@ impl Expression {
             Expression::ReadLocalVariable { .. } => {}
             Expression::EasingCurve(_) => {}
             Expression::LinearGradient { angle, stops } => {
-                visitor(&angle);
+                visitor(angle);
                 for (c, s) in stops {
                     visitor(c);
                     visitor(s);
@@ -848,40 +859,41 @@ impl Expression {
                     rhs: Box::new(Expression::NumberLiteral(0.01, Unit::None)),
                     op: '*',
                 },
-                (Type::Struct { fields: ref a, .. }, Type::Struct { fields: b, name, node: n })
-                    if a != b =>
-                {
+                (
+                    Type::Struct { fields: ref left, .. },
+                    Type::Struct { fields: right, name, node: n },
+                ) if left != right => {
                     if let Expression::Struct { mut values, .. } = self {
                         let mut new_values = HashMap::new();
-                        for (k, ty) in b {
-                            let (k, e) = values.remove_entry(k).map_or_else(
-                                || (k.clone(), Expression::default_value_for_type(ty)),
+                        for (key, ty) in right {
+                            let (key, expression) = values.remove_entry(key).map_or_else(
+                                || (key.clone(), Expression::default_value_for_type(ty)),
                                 |(k, e)| (k, e.maybe_convert_to(ty.clone(), node, diag)),
                             );
-                            new_values.insert(k, e);
+                            new_values.insert(key, expression);
                         }
                         return Expression::Struct { values: new_values, ty: target_type };
                     }
                     let var_name = "tmpobj";
                     let mut new_values = HashMap::new();
-                    for (k, ty) in b {
-                        let e = if a.contains_key(k) {
+                    for (key, ty) in right {
+                        let expression = if left.contains_key(key) {
                             Expression::StructFieldAccess {
                                 base: Box::new(Expression::ReadLocalVariable {
                                     name: var_name.into(),
                                     ty: Type::Struct {
-                                        fields: a.clone(),
+                                        fields: left.clone(),
                                         name: name.clone(),
                                         node: n.clone(),
                                     },
                                 }),
-                                name: k.clone(),
+                                name: key.clone(),
                             }
                             .maybe_convert_to(ty.clone(), node, diag)
                         } else {
                             Expression::default_value_for_type(ty)
                         };
-                        new_values.insert(k.clone(), e);
+                        new_values.insert(key.clone(), expression);
                     }
                     return Expression::CodeBlock(vec![
                         Expression::StoreLocalVariable {
@@ -891,9 +903,9 @@ impl Expression {
                         Expression::Struct { values: new_values, ty: target_type },
                     ]);
                 }
-                (Type::Struct { .. }, Type::Component(c)) => {
+                (Type::Struct { .. }, Type::Component(component)) => {
                     let struct_type_for_component = Type::Struct {
-                        fields: c
+                        fields: component
                             .root_element
                             .borrow()
                             .property_declarations
@@ -907,9 +919,10 @@ impl Expression {
                     };
                     self.maybe_convert_to(struct_type_for_component, node, diag)
                 }
-                (a, b) => match (a.as_unit_product(), b.as_unit_product()) {
-                    (Some(a), Some(b)) => {
-                        if let Some(power) = crate::langtype::unit_product_length_conversion(&a, &b)
+                (left, right) => match (left.as_unit_product(), right.as_unit_product()) {
+                    (Some(left), Some(right)) => {
+                        if let Some(power) =
+                            crate::langtype::unit_product_length_conversion(&left, &right)
                         {
                             let op = if power < 0 { '*' } else { '/' };
                             let mut result = self;
@@ -936,8 +949,8 @@ impl Expression {
                 },
             };
             Expression::Cast { from: Box::new(from), to: target_type }
-        } else if matches!((&ty, &target_type, &self), (Type::Array(a), Type::Array(b), Expression::Array{..})
-            if a.can_convert(b) || **a == Type::Invalid)
+        } else if matches!((&ty, &target_type, &self), (Type::Array(left), Type::Array(right), Expression::Array{..})
+            if left.can_convert(right) || **left == Type::Invalid)
         {
             // Special case for converting array literals
             match (self, target_type) {
@@ -956,7 +969,7 @@ impl Expression {
             if let Some(from_unit) = ty.default_unit() {
                 if matches!(&target_type, Type::Int32 | Type::Float32 | Type::String) {
                     message = format!(
-                        "{}. Divide by 1{} to convert to a plain number.",
+                        "{}. Divide by 1{} to convert to a plain number",
                         message, from_unit
                     );
                 }
@@ -994,11 +1007,10 @@ impl Expression {
             | Type::ElementReference
             | Type::LayoutCache => Expression::Invalid,
             Type::Float32 => Expression::NumberLiteral(0., Unit::None),
-            Type::Int32 => Expression::NumberLiteral(0., Unit::None),
             Type::String => Expression::StringLiteral(String::new()),
-            Type::Color => Expression::Cast {
+            Type::Int32 | Type::Color | Type::UnitProduct(_) => Expression::Cast {
                 from: Box::new(Expression::NumberLiteral(0., Unit::None)),
-                to: Type::Color,
+                to: ty.clone(),
             },
             Type::Duration => Expression::NumberLiteral(0., Unit::Ms),
             Type::Angle => Expression::NumberLiteral(0., Unit::Deg),
@@ -1028,10 +1040,6 @@ impl Expression {
             Type::Enumeration(enumeration) => {
                 Expression::EnumerationValue(enumeration.clone().default_value())
             }
-            Type::UnitProduct(_) => Expression::Cast {
-                from: Box::new(Expression::NumberLiteral(0., Unit::None)),
-                to: ty.clone(),
-            },
         }
     }
 
@@ -1071,13 +1079,21 @@ pub struct BindingExpression {
     /// 0 means the expression was added by some passes and it is not explicit in the source code
     pub priority: i32,
 
+    pub animation: Option<PropertyAnimation>,
+
     /// The analysis information. None before it is computed
     pub analysis: RefCell<Option<BindingAnalysis>>,
 }
 
 impl std::convert::From<Expression> for BindingExpression {
     fn from(expression: Expression) -> Self {
-        Self { expression, span: None, priority: 0, analysis: Default::default() }
+        Self {
+            expression,
+            span: None,
+            priority: 0,
+            animation: Default::default(),
+            analysis: Default::default(),
+        }
     }
 }
 
@@ -1087,11 +1103,59 @@ impl BindingExpression {
             expression: Expression::Uncompiled(node.clone()),
             span: Some(node.to_source_location()),
             priority: 1,
+            animation: Default::default(),
             analysis: Default::default(),
         }
     }
     pub fn new_with_span(expression: Expression, span: SourceLocation) -> Self {
-        Self { expression, span: Some(span), priority: 0, analysis: Default::default() }
+        Self {
+            expression,
+            span: Some(span),
+            priority: 0,
+            animation: Default::default(),
+            analysis: Default::default(),
+        }
+    }
+
+    /// Merge the other into this one. Normally, &self is kept intact (has priority), unless two ways binding are
+    /// involved then they need to be merged.
+    /// Also the animation is taken if the other don't have one
+    ///
+    /// Returns true if the other expression was taken
+    pub fn merge_with(&mut self, other: &Self) -> bool {
+        fn maybe_merge_two_ways(
+            binding: &mut Expression,
+            priority: &mut i32,
+            original: &BindingExpression,
+        ) -> bool {
+            match (binding, &original.expression) {
+                (Expression::TwoWayBinding(_, Some(ref mut x)), _) => {
+                    maybe_merge_two_ways(&mut *x, priority, original)
+                }
+                (Expression::TwoWayBinding(_, x), o) => {
+                    *priority = original.priority;
+                    *x = Some(Box::new(o.clone()));
+                    true
+                }
+                (ref mut x, Expression::TwoWayBinding(nr, _)) => {
+                    let n =
+                        Expression::TwoWayBinding(nr.clone(), Some(Box::new(std::mem::take(*x))));
+                    **x = n;
+                    false
+                }
+                _ => false,
+            }
+        }
+        if self.animation.is_none() {
+            self.animation = other.animation.clone();
+        }
+        if matches!(self.expression, Expression::Invalid) {
+            self.priority = other.priority;
+            self.expression = other.expression.clone();
+            true
+        } else {
+            maybe_merge_two_ways(&mut self.expression, &mut self.priority, other)
+        }
     }
 }
 
@@ -1113,7 +1177,8 @@ pub struct BindingAnalysis {
     pub is_const: bool,
 }
 
-pub type PathEvents = Vec<lyon_path::Event<lyon_path::math::Point, lyon_path::math::Point>>;
+pub type PathEvent = lyon_path::Event<lyon_path::math::Point, lyon_path::math::Point>;
+pub type PathEvents = Vec<PathEvent>;
 
 #[derive(Debug, Clone)]
 pub enum Path {
@@ -1124,7 +1189,7 @@ pub enum Path {
 #[derive(Debug, Clone)]
 pub struct PathElement {
     pub element_type: Rc<BuiltinElement>,
-    pub bindings: BTreeMap<String, BindingExpression>,
+    pub bindings: BindingsMap,
 }
 
 #[derive(Clone, Debug)]
@@ -1147,7 +1212,7 @@ impl Default for EasingCurve {
 pub enum ImageReference {
     None,
     AbsolutePath(String),
-    EmbeddedData(usize),
+    EmbeddedData { resource_id: usize, extension: String },
 }
 
 /// Print the expression as a .60 code (not necessarily valid .60)
@@ -1265,12 +1330,12 @@ pub fn pretty_print(f: &mut dyn std::fmt::Write, expression: &Expression) -> std
         Expression::EasingCurve(e) => write!(f, "{:?}", e),
         Expression::LinearGradient { angle, stops } => {
             write!(f, "@linear-gradient(")?;
-            pretty_print(f, &angle)?;
+            pretty_print(f, angle)?;
             for (c, s) in stops {
                 write!(f, ", ")?;
-                pretty_print(f, &c)?;
+                pretty_print(f, c)?;
                 write!(f, "  ")?;
-                pretty_print(f, &s)?;
+                pretty_print(f, s)?;
             }
             write!(f, ")")
         }
