@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -18,6 +18,7 @@ use std::rc::{Rc, Weak};
 
 use crate::diagnostics::BuildDiagnostics;
 use crate::expression_tree::{BindingExpression, Expression};
+use crate::langtype::Type;
 use crate::namedreference::NamedReference;
 use crate::object_tree::{Component, Document, ElementRc};
 
@@ -69,6 +70,12 @@ pub fn generate(
 ) -> std::io::Result<()> {
     #![allow(unused_variables)]
     #![allow(unreachable_code)]
+
+    if matches!(doc.root_component.root_element.borrow().base_type, Type::Invalid | Type::Void) {
+        // empty document, nothing to generate
+        return Ok(());
+    }
+
     match format {
         #[cfg(feature = "cpp")]
         OutputFormat::Cpp => {
@@ -100,7 +107,7 @@ pub fn generate(
 #[allow(dead_code)]
 pub fn build_array_helper(component: &Component, mut visit_item: impl FnMut(&ElementRc, u32, u32)) {
     visit_item(&component.root_element, 1, 0);
-    visit_children(&component.root_element, &mut 0, 1, &mut visit_item);
+    visit_children(&component.root_element, 0, 1, &mut visit_item);
 
     fn sub_children_count(e: &ElementRc) -> usize {
         let mut count = e.borrow().children.len();
@@ -112,23 +119,24 @@ pub fn build_array_helper(component: &Component, mut visit_item: impl FnMut(&Ele
 
     fn visit_children(
         item: &ElementRc,
-        index: &mut u32,
+        index: u32,
         children_offset: u32,
         visit_item: &mut impl FnMut(&ElementRc, u32, u32),
     ) {
+        debug_assert_eq!(index, item.borrow().item_index.get().map(|x| *x as u32).unwrap_or(index));
         let mut offset = children_offset + item.borrow().children.len() as u32;
 
         for i in &item.borrow().children {
-            visit_item(i, offset, *index);
+            visit_item(i, offset, index);
             offset += sub_children_count(i) as u32;
         }
 
-        *index += 1;
-
         let mut offset = children_offset + item.borrow().children.len() as u32;
+        let mut index = children_offset;
 
         for e in &item.borrow().children {
             visit_children(e, index, offset, visit_item);
+            index += 1;
             offset += sub_children_count(e) as u32;
         }
     }
@@ -156,8 +164,9 @@ pub fn handle_property_bindings_init(
         processed.insert(nr);
         if binding_expression.analysis.borrow().as_ref().map_or(false, |a| a.is_const) {
             // We must first handle all dependent properties in case it is a constant property
-            binding_expression.expression.visit_recursive(&mut |e| match e {
-                Expression::PropertyReference(nr) => {
+
+            binding_expression.expression.visit_recursive(&mut |e| {
+                if let Expression::PropertyReference(nr) = e {
                     let elem = nr.element();
                     if Weak::ptr_eq(&elem.borrow().enclosing_component, component) {
                         if let Some(be) = elem.borrow().bindings.get(nr.name()) {
@@ -172,9 +181,11 @@ pub fn handle_property_bindings_init(
                         }
                     }
                 }
-                _ => {}
             })
         }
+        if matches!(binding_expression.expression, Expression::Invalid) {
+            return;
+        }
         handle_property(elem, prop_name, binding_expression);
     }
 
