@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -10,6 +10,8 @@ LICENSE END */
 /*! module for the C++ code generator
 */
 
+// cSpell:ignore cstdlib cmath constexpr nullptr decltype intptr uintptr
+
 /// This module contains some data structure that helps represent a C++ code.
 /// It is then rendered into an actual C++ text using the Display trait
 mod cpp_ast {
@@ -150,7 +152,7 @@ mod cpp_ast {
             if self.is_friend {
                 write!(f, "friend ")?;
             }
-            // all functions are inlines because we are in a header
+            // all functions are `inline` because we are in a header
             write!(f, "inline ")?;
             if !self.is_constructor_or_destructor {
                 write!(f, "auto ")?;
@@ -223,13 +225,22 @@ use crate::expression_tree::{
 use crate::langtype::Type;
 use crate::layout::{Layout, LayoutGeometry, LayoutRect, Orientation};
 use crate::object_tree::{
-    Component, Document, Element, ElementRc, PropertyDeclaration, RepeatedElementInfo,
+    Component, Document, ElementRc, PropertyAnimation, PropertyDeclaration, RepeatedElementInfo,
 };
 use cpp_ast::*;
 use itertools::Itertools;
+use std::borrow::Cow;
 use std::collections::BTreeMap;
 use std::rc::Rc;
 
+fn ident(ident: &str) -> Cow<'_, str> {
+    if ident.contains('-') {
+        ident.replace('-', "_").into()
+    } else {
+        ident.into()
+    }
+}
+
 impl CppType for Type {
     fn cpp_type(&self) -> Option<String> {
         match self {
@@ -244,9 +255,9 @@ impl CppType for Type {
             Type::LogicalLength => Some("float".to_owned()),
             Type::Percent => Some("float".to_owned()),
             Type::Bool => Some("bool".to_owned()),
-            Type::Struct { name: Some(name), node: Some(_), .. } => Some(name.clone()),
+            Type::Struct { name: Some(name), node: Some(_), .. } => Some(ident(name).into_owned()),
             Type::Struct { name: Some(name), node: None, .. } => {
-                Some(format!("sixtyfps::cbindgen_private::{}", name))
+                Some(format!("sixtyfps::cbindgen_private::{}", ident(name)))
             }
             Type::Struct { fields, .. } => {
                 let elem = fields.values().map(|v| v.cpp_type()).collect::<Option<Vec<_>>>()?;
@@ -258,7 +269,7 @@ impl CppType for Type {
             Type::Image => Some("sixtyfps::Image".to_owned()),
             Type::Builtin(elem) => elem.native_class.cpp_type.clone(),
             Type::Enumeration(enumeration) => {
-                Some(format!("sixtyfps::cbindgen_private::{}", enumeration.name))
+                Some(format!("sixtyfps::cbindgen_private::{}", ident(&enumeration.name)))
             }
             Type::Brush => Some("sixtyfps::Brush".to_owned()),
             Type::LayoutCache => Some("sixtyfps::SharedVector<float>".into()),
@@ -299,7 +310,7 @@ fn new_struct_with_bindings(
         .iter()
         .map(|(prop, initializer)| {
             let initializer = compile_expression(initializer, component);
-            format!("var.{} = {};", prop, initializer)
+            format!("var.{} = {};", ident(prop), initializer)
         })
         .collect();
 
@@ -324,11 +335,11 @@ fn property_animation_code(component: &Rc<Component>, animation: &ElementRc) ->
 
 fn property_set_value_code(
     component: &Rc<Component>,
-    element: &Element,
+    element: &ElementRc,
     property_name: &str,
     value_expr: &str,
 ) -> String {
-    match element.property_animations.get(property_name) {
+    match element.borrow().bindings.get(property_name).and_then(|b| b.animation.as_ref()) {
         Some(crate::object_tree::PropertyAnimation::Static(animation)) => {
             let animation_code = property_animation_code(component, animation);
             format!(
@@ -346,6 +357,7 @@ fn handle_property_binding(
     prop_name: &str,
     binding_expression: &Expression,
     is_constant: bool,
+    animation: Option<&PropertyAnimation>,
     init: &mut Vec<String>,
 ) {
     let item = elem.borrow();
@@ -355,10 +367,10 @@ fn handle_property_binding(
     } else if item.is_flickable_viewport {
         format!(
             "{id}.viewport.",
-            id = crate::object_tree::find_parent_element(elem).unwrap().borrow().id
+            id = ident(&crate::object_tree::find_parent_element(elem).unwrap().borrow().id)
         )
     } else {
-        format!("{id}.", id = item.id)
+        format!("{id}.", id = ident(&item.id))
     };
     let prop_type = item.lookup_property(prop_name).property_type;
     if let Type::Callback { args, .. } = &prop_type {
@@ -373,7 +385,7 @@ fn handle_property_binding(
                         return {code};
                     }});",
             accessor_prefix = accessor_prefix,
-            prop = prop_name,
+            prop = ident(prop_name),
             params = params.join(", "),
             code = compile_expression_wrap_return(binding_expression, &component)
         ));
@@ -385,13 +397,20 @@ fn handle_property_binding(
             p2 = access_named_reference(nr, &component, "this")
         ));
         if let Some(next) = next {
-            handle_property_binding(elem, prop_name, next, is_constant || next.is_constant(), init)
+            handle_property_binding(
+                elem,
+                prop_name,
+                next,
+                is_constant || next.is_constant(),
+                animation,
+                init,
+            )
         }
     } else {
         let component = &item.enclosing_component.upgrade().unwrap();
 
         let init_expr = compile_expression_wrap_return(binding_expression, component);
-        let cpp_prop = format!("{}{}", accessor_prefix, prop_name);
+        let cpp_prop = format!("{}{}", accessor_prefix, ident(prop_name));
 
         init.push(if is_constant {
             format!("{}.set({});", cpp_prop, init_expr)
@@ -408,9 +427,9 @@ fn handle_property_binding(
             if is_state_info {
                 format!("sixtyfps::private_api::set_state_binding({}, {});", cpp_prop, binding_code)
             } else {
-                match item.property_animations.get(prop_name) {
+                match animation {
                     Some(crate::object_tree::PropertyAnimation::Static(anim)) => {
-                        let anim = property_animation_code(&component, anim);
+                        let anim = property_animation_code(component, anim);
                         format!("{}.set_animated_binding({}, {});", cpp_prop, binding_code, anim)
                     }
                     Some(crate::object_tree::PropertyAnimation::Transition {
@@ -456,8 +475,11 @@ fn handle_item(elem: &ElementRc, main_struct: &mut Struct) {
     main_struct.members.push((
         Access::Private,
         Declaration::Var(Var {
-            ty: format!("sixtyfps::cbindgen_private::{}", item.base_type.as_native().class_name),
-            name: item.id.clone(),
+            ty: format!(
+                "sixtyfps::cbindgen_private::{}",
+                ident(&item.base_type.as_native().class_name)
+            ),
+            name: ident(&item.id).into_owned(),
             init: Some("{}".to_owned()),
         }),
     ));
@@ -476,7 +498,7 @@ fn handle_repeater(
     diag: &mut BuildDiagnostics,
 ) {
     let parent_element = base_component.parent_element.upgrade().unwrap();
-    let repeater_id = format!("repeater_{}", parent_element.borrow().id);
+    let repeater_id = format!("repeater_{}", ident(&parent_element.borrow().id));
 
     let mut model = compile_expression(&repeated.model, parent_component);
     if repeated.is_conditional_element {
@@ -558,12 +580,12 @@ pub fn generate(doc: &Document, diag: &mut BuildDiagnostics) -> Option<impl std:
     file.includes.push("<cmath>".into()); // TODO: ideally only include this if needed (by floor/ceil/round)
     file.includes.push("<sixtyfps.h>".into());
 
-    for ty in doc.root_component.used_structs.borrow().iter() {
+    for ty in doc.root_component.used_types.borrow().structs.iter() {
         if let Type::Struct { fields, name: Some(name), node: Some(_) } = ty {
             generate_struct(&mut file, name, fields, diag);
         }
     }
-    for glob in doc.root_component.used_global.borrow().iter() {
+    for glob in doc.root_component.used_types.borrow().globals.iter() {
         if !matches!(glob.root_element.borrow().base_type, Type::Builtin(_)) {
             generate_component(&mut file, glob, diag, None);
         }
@@ -599,7 +621,7 @@ fn generate_struct(
         .iter()
         .map(|(name, t)| {
             use std::fmt::Write;
-            write!(operator_eq, " && a.{0} == b.{0}", name).unwrap();
+            write!(operator_eq, " && a.{0} == b.{0}", ident(name)).unwrap();
             (
                 Access::Public,
                 Declaration::Var(Var {
@@ -610,7 +632,7 @@ fn generate_struct(
                         );
                         Default::default()
                     }),
-                    name: name.clone(),
+                    name: ident(name).into_owned(),
                     ..Default::default()
                 }),
             )
@@ -671,7 +693,8 @@ fn generate_component(
     let is_root = component.parent_element.upgrade().is_none();
     let mut init = vec!["[[maybe_unused]] auto self = this;".into()];
 
-    for (cpp_name, property_decl) in component.root_element.borrow().property_declarations.iter() {
+    for (prop_name, property_decl) in component.root_element.borrow().property_declarations.iter() {
+        let cpp_name = ident(prop_name);
         let access = if let Some(alias) = &property_decl.is_alias {
             access_named_reference(alias, component, "this")
         } else {
@@ -683,7 +706,7 @@ fn generate_component(
                 args.iter().map(|t| get_cpp_type(t, property_decl, diag)).collect::<Vec<_>>();
             let return_type = return_type
                 .as_ref()
-                .map_or("void".into(), |t| get_cpp_type(&t, property_decl, diag));
+                .map_or("void".into(), |t| get_cpp_type(t, property_decl, diag));
             if property_decl.expose_in_public_api && is_root {
                 let callback_emitter = vec![format!(
                     "return {}.call({});",
@@ -737,18 +760,15 @@ fn generate_component(
                     }),
                 ));
 
+                let set_value = if let Some(alias) = &property_decl.is_alias {
+                    property_set_value_code(component, &alias.element(), alias.name(), "value")
+                } else {
+                    property_set_value_code(component, &component.root_element, prop_name, "value")
+                };
+
                 let prop_setter: Vec<String> = vec![
                     "[[maybe_unused]] auto self = this;".into(),
-                    format!(
-                        "{}.{};",
-                        access,
-                        property_set_value_code(
-                            &component,
-                            &*component.root_element.borrow(),
-                            cpp_name,
-                            "value"
-                        )
-                    ),
+                    format!("{}.{};", access, set_value),
                 ];
                 component_struct.members.push((
                     Access::Public,
@@ -766,7 +786,7 @@ fn generate_component(
         if property_decl.is_alias.is_none() {
             component_struct.members.push((
                 if component.is_global() { Access::Public } else { Access::Private },
-                Declaration::Var(Var { ty, name: cpp_name.clone(), init: None }),
+                Declaration::Var(Var { ty, name: cpp_name.into_owned(), init: None }),
             ));
         }
     }
@@ -893,18 +913,18 @@ fn generate_component(
         component_struct.members.push((
             Access::Private,
             Declaration::Var(Var {
-                ty: "sixtyfps::private_api::ComponentWindow".into(),
-                name: "window".into(),
-                ..Var::default()
+                ty: "sixtyfps::Window".into(),
+                name: "m_window".into(),
+                init: Some("sixtyfps::Window{sixtyfps::private_api::WindowRc()}".into()),
             }),
         ));
     } else if !component.is_global() {
         component_struct.members.push((
             Access::Public, // FIXME: many of the different component bindings need to access this
             Declaration::Var(Var {
-                ty: "sixtyfps::private_api::ComponentWindow".into(),
-                name: "window".into(),
-                ..Var::default()
+                ty: "sixtyfps::Window".into(),
+                name: "m_window".into(),
+                init: Some("sixtyfps::Window{sixtyfps::private_api::WindowRc()}".into()),
             }),
         ));
 
@@ -924,8 +944,8 @@ fn generate_component(
             Access::Public,
             Declaration::Function(Function {
                 name: "show".into(),
-                signature: "() const".into(),
-                statements: Some(vec!["window.show();".into()]),
+                signature: "()".into(),
+                statements: Some(vec!["m_window.show();".into()]),
                 ..Default::default()
             }),
         ));
@@ -934,8 +954,21 @@ fn generate_component(
             Access::Public,
             Declaration::Function(Function {
                 name: "hide".into(),
-                signature: "() const".into(),
-                statements: Some(vec!["window.hide();".into()]),
+                signature: "()".into(),
+                statements: Some(vec!["m_window.hide();".into()]),
+                ..Default::default()
+            }),
+        ));
+
+        component_struct.members.push((
+            Access::Public,
+            Declaration::Function(Function {
+                name: "window".into(),
+                signature: "() const -> sixtyfps::Window&".into(),
+                statements: Some(vec![format!(
+                    "return const_cast<{} *>(this)->m_window;",
+                    component_struct.name
+                )]),
                 ..Default::default()
             }),
         ));
@@ -944,7 +977,7 @@ fn generate_component(
             Access::Public,
             Declaration::Function(Function {
                 name: "run".into(),
-                signature: "() const".into(),
+                signature: "()".into(),
                 statements: Some(vec![
                     "show();".into(),
                     "sixtyfps::run_event_loop();".into(),
@@ -954,9 +987,9 @@ fn generate_component(
             }),
         ));
 
-        init.push("self->window.init_items(this, item_tree());".into());
+        init.push("self->m_window.window_handle().init_items(this, item_tree());".into());
 
-        component_struct.friends.push("sixtyfps::private_api::ComponentWindow".into());
+        component_struct.friends.push("sixtyfps::private_api::WindowRc".into());
     }
 
     if !component.is_global() {
@@ -969,7 +1002,9 @@ fn generate_component(
         ];
 
         if component.parent_element.upgrade().is_none() {
-            create_code.push("self->window.set_component(**self->self_weak.lock());".into());
+            create_code.push(
+                "self->m_window.window_handle().set_component(**self->self_weak.lock());".into(),
+            );
         }
 
         create_code.extend(
@@ -1035,7 +1070,7 @@ fn generate_component(
                 tree_array.push(format!(
                     "sixtyfps::private_api::make_item_node(offsetof({}, {}) + offsetof(sixtyfps::cbindgen_private::Flickable, viewport), SIXTYFPS_GET_ITEM_VTABLE(RectangleVTable), {}, {}, {})",
                     &component_id,
-                    crate::object_tree::find_parent_element(item_rc).unwrap().borrow().id,
+                    ident(&crate::object_tree::find_parent_element(item_rc).unwrap().borrow().id),
                     item.children.len(),
                     children_offset,
                     parent_index,
@@ -1044,7 +1079,7 @@ fn generate_component(
                 tree_array.push(format!(
                     "sixtyfps::private_api::make_item_node(offsetof({}, {}), {}, {}, {}, {})",
                     component_id,
-                    item.id,
+                    ident(&item.id),
                     item.base_type.as_native().cpp_vtable_getter,
                     item.children.len(),
                     children_offset,
@@ -1052,8 +1087,10 @@ fn generate_component(
                 ));
             }
             handle_item(item_rc, &mut component_struct);
-            item_names_and_vt_symbols
-                .push((item.id.clone(), item.base_type.as_native().cpp_vtable_getter.clone()));
+            item_names_and_vt_symbols.push((
+                ident(&item.id).into_owned(),
+                item.base_type.as_native().cpp_vtable_getter.clone(),
+            ));
         }
     });
 
@@ -1063,6 +1100,7 @@ fn generate_component(
             prop,
             binding,
             binding.analysis.borrow().as_ref().map_or(false, |a| a.is_const),
+            binding.animation.as_ref(),
             &mut init,
         )
     });
@@ -1080,7 +1118,7 @@ fn generate_component(
             is_constructor_or_destructor: true,
             statements: Some(init),
             constructor_member_initializers: if !component.is_global() && !is_root {
-                vec!["window(parent->window)".into()]
+                vec!["m_window(parent->m_window.window_handle())".into()]
             } else {
                 vec![]
             },
@@ -1089,9 +1127,8 @@ fn generate_component(
     ));
 
     if !component.is_global() {
-        let mut destructor = Vec::new();
+        let mut destructor = vec!["[[maybe_unused]] auto self = this;".to_owned()];
 
-        destructor.push("[[maybe_unused]] auto self = this;".to_owned());
         if component.parent_element.upgrade().is_some() {
             destructor.push("if (!parent) return;".to_owned())
         }
@@ -1110,7 +1147,7 @@ fn generate_component(
                     .join(","),
             );
             destructor.push("};".into());
-            destructor.push("window.free_graphics_resources(sixtyfps::Slice<sixtyfps::private_api::ItemRef>{items, std::size(items)});".into());
+            destructor.push("m_window.window_handle().free_graphics_resources(sixtyfps::Slice<sixtyfps::private_api::ItemRef>{items, std::size(items)});".into());
         }
 
         component_struct.members.push((
@@ -1158,7 +1195,7 @@ fn generate_component(
         ));
 
         let parent_item_from_parent_component = if let Some(parent_index) =
-            component.parent_element.upgrade().and_then(|e| e.borrow().item_index.get().map(|x| *x))
+            component.parent_element.upgrade().and_then(|e| e.borrow().item_index.get().copied())
         {
             format!(
                 "   *result = sixtyfps::private_api::parent_item(self->parent->self_weak.into_dyn(), self->parent->item_tree(), {});",
@@ -1239,14 +1276,14 @@ fn generate_component(
                 statements: Some(vec![format!(
                     "return {{ {vt}, const_cast<decltype(this->{id})*>(&this->{id}) }};",
                     vt = root_elem.base_type.as_native().cpp_vtable_getter,
-                    id = root_elem.id
+                    id = ident(&root_elem.id)
                 )]),
                 ..Default::default()
             }),
         ));
     }
 
-    for glob in component.used_global.borrow().iter() {
+    for glob in component.used_types.borrow().globals.iter() {
         let ty = match &glob.root_element.borrow().base_type {
             Type::Void => self::component_id(glob),
             Type::Builtin(b) => {
@@ -1258,7 +1295,7 @@ fn generate_component(
             Access::Private,
             Declaration::Var(Var {
                 ty: format!("std::shared_ptr<{}>", ty),
-                name: format!("global_{}", glob.id),
+                name: format!("global_{}", self::component_id(glob)),
                 init: Some(format!("std::make_shared<{}>()", ty)),
             }),
         ));
@@ -1280,10 +1317,12 @@ fn generate_component(
 }
 
 fn component_id(component: &Rc<Component>) -> String {
-    if component.id.is_empty() {
-        format!("Component_{}", component.root_element.borrow().id)
+    if component.is_global() {
+        ident(&component.root_element.borrow().id).into_owned()
+    } else if component.id.is_empty() {
+        format!("Component_{}", ident(&component.root_element.borrow().id))
     } else {
-        component.id.clone()
+        ident(&component.id).into_owned()
     }
 }
 
@@ -1326,17 +1365,17 @@ fn access_member(
     let e = element.borrow();
     let enclosing_component = e.enclosing_component.upgrade().unwrap();
     if Rc::ptr_eq(component, &enclosing_component) {
-        if e.property_declarations.contains_key(name) || name == "" || component.is_global() {
-            format!("{}->{}", component_cpp, name)
+        if e.property_declarations.contains_key(name) || name.is_empty() || component.is_global() {
+            format!("{}->{}", component_cpp, ident(name))
         } else if e.is_flickable_viewport {
             format!(
                 "{}->{}.viewport.{}",
                 component_cpp,
-                crate::object_tree::find_parent_element(element).unwrap().borrow().id,
-                name
+                ident(&crate::object_tree::find_parent_element(element).unwrap().borrow().id),
+                ident(name)
             )
         } else {
-            format!("{}->{}.{}", component_cpp, e.id.as_str(), name)
+            format!("{}->{}.{}", component_cpp, ident(&e.id), ident(name))
         }
     } else if enclosing_component.is_global() {
         let mut root_component = component.clone();
@@ -1345,7 +1384,8 @@ fn access_member(
             root_component = p.borrow().enclosing_component.upgrade().unwrap();
             component_cpp = format!("{}->parent", component_cpp);
         }
-        let global_comp = format!("{}->global_{}", component_cpp, enclosing_component.id);
+        let global_comp =
+            format!("{}->global_{}", component_cpp, component_id(&enclosing_component));
         access_member(element, name, &enclosing_component, &global_comp)
     } else {
         access_member(
@@ -1394,7 +1434,7 @@ fn compile_expression(
         ),
         Expression::BuiltinFunctionReference(funcref, _) => match funcref {
             BuiltinFunction::GetWindowScaleFactor => {
-                "self->window.scale_factor".into()
+                "self->m_window.window_handle().scale_factor".into()
             }
             BuiltinFunction::Debug => {
                 "[](auto... args){ (std::cout << ... << args) << std::endl; return nullptr; }"
@@ -1413,10 +1453,10 @@ fn compile_expression(
             BuiltinFunction::ACos => format!("[](float a){{ return std::acos(a) / {}; }}", std::f32::consts::PI / 180.),
             BuiltinFunction::ATan => format!("[](float a){{ return std::atan(a) / {}; }}", std::f32::consts::PI / 180.),
             BuiltinFunction::SetFocusItem => {
-                "self->window.set_focus_item".into()
+                "self->m_window.window_handle().set_focus_item".into()
             }
             BuiltinFunction::ShowPopupWindow => {
-                "self->window.show_popup".into()
+                "self->m_window.window_handle().show_popup".into()
             }
 
            /*  std::from_chars is unfortunately not yet implemented in gcc
@@ -1481,9 +1521,9 @@ fn compile_expression(
         }
         Expression::FunctionParameterReference { index, .. } => format!("arg_{}", index),
         Expression::StoreLocalVariable { name, value } => {
-            format!("auto {} = {};", name, compile_expression(value, component))
+            format!("auto {} = {};", ident(name), compile_expression(value, component))
         }
-        Expression::ReadLocalVariable { name, .. } => name.clone(),
+        Expression::ReadLocalVariable { name, .. } => ident(name).into_owned(),
         Expression::StructFieldAccess { base, name } => match base.ty() {
             Type::Struct { fields, name : None, .. } => {
                 let index = fields
@@ -1493,7 +1533,7 @@ fn compile_expression(
                 format!("std::get<{}>({})", index, compile_expression(base, component))
             }
             Type::Struct{..} => {
-                format!("{}.{}", compile_expression(base, component), name)
+                format!("{}.{}", compile_expression(base, component), ident(name))
             }
             _ => panic!("Expression::ObjectAccess's base expression is not an Object type"),
         },
@@ -1552,7 +1592,7 @@ fn compile_expression(
                 if let Expression::ElementReference(focus_item) = &arguments[0] {
                     let focus_item = focus_item.upgrade().unwrap();
                     let focus_item = focus_item.borrow();
-                    format!("self->window.set_focus_item(self->self_weak.lock()->into_dyn(), {});", focus_item.item_index.get().unwrap())
+                    format!("self->m_window.window_handle().set_focus_item(self->self_weak.lock()->into_dyn(), {});", focus_item.item_index.get().unwrap())
                 } else {
                     panic!("internal error: argument to SetFocusItem must be an element")
                 }
@@ -1564,13 +1604,13 @@ fn compile_expression(
                 if let Expression::ElementReference(popup_window) = &arguments[0] {
                     let popup_window = popup_window.upgrade().unwrap();
                     let pop_comp = popup_window.borrow().enclosing_component.upgrade().unwrap();
-                    let popup_window_id = component_id(&pop_comp);
+                    let popup_window_rcid = component_id(&pop_comp);
                     let parent_component = pop_comp.parent_element.upgrade().unwrap().borrow().enclosing_component.upgrade().unwrap();
                     let popup_list = parent_component.popup_windows.borrow();
                     let popup = popup_list.iter().find(|p| Rc::ptr_eq(&p.component, &pop_comp)).unwrap();
                     let x = access_named_reference(&popup.x, component, "self");
                     let y = access_named_reference(&popup.y, component, "self");
-                    format!("self->window.show_popup<{}>(self, {{ {}.get(), {}.get() }} );", popup_window_id, x, y)
+                    format!("self->m_window.window_handle().show_popup<{}>(self, {{ {}.get(), {}.get() }} );", popup_window_rcid, x, y)
                 } else {
                     panic!("internal error: argument to SetFocusItem must be an element")
                 }
@@ -1583,10 +1623,10 @@ fn compile_expression(
                     let item = item.upgrade().unwrap();
                     let item = item.borrow();
                     let native_item = item.base_type.as_native();
-                    format!("{vt}->layouting_info({{{vt}, const_cast<sixtyfps::cbindgen_private::{ty}*>(&self->{id})}}, {o}, &window)",
+                    format!("{vt}->layouting_info({{{vt}, const_cast<sixtyfps::cbindgen_private::{ty}*>(&self->{id})}}, {o}, &m_window.window_handle())",
                         vt = native_item.cpp_vtable_getter,
                         ty = native_item.class_name,
-                        id = item.id,
+                        id = ident(&item.id),
                         o = to_cpp_orientation(*orientation),
                     )
                 } else {
@@ -1598,7 +1638,7 @@ fn compile_expression(
                     panic!("internal error: incorrect argument count to RegisterCustomFontByPath call");
                 }
                 if let Expression::StringLiteral(font_path) = &arguments[0] {
-                    format!("sixtyfps::private_api::register_font_from_path(\"{}\");", font_path)
+                    format!("sixtyfps::private_api::register_font_from_path(\"{}\");", escape_string(font_path))
                 } else {
                     panic!("internal error: argument to RegisterCustomFontByPath must be a string literal")
                 }
@@ -1606,11 +1646,11 @@ fn compile_expression(
             _ => {
                 let mut args = arguments.iter().map(|e| compile_expression(e, component));
 
-                format!("{}({})", compile_expression(&function, component), args.join(", "))
+                format!("{}({})", compile_expression(function, component), args.join(", "))
             }
         },
         Expression::SelfAssignment { lhs, rhs, op } => {
-            let rhs = compile_expression(&*rhs, &component);
+            let rhs = compile_expression(&*rhs, component);
             compile_assignment(lhs, *op, rhs, component)
         }
         Expression::BinaryExpression { lhs, rhs, op } => {
@@ -1636,9 +1676,9 @@ fn compile_expression(
         }
         Expression::ImageReference(resource_ref)  => {
             match resource_ref {
-                crate::expression_tree::ImageReference::None => format!(r#"sixtyfps::Image()"#),
+                crate::expression_tree::ImageReference::None => r#"sixtyfps::Image()"#.to_string(),
                 crate::expression_tree::ImageReference::AbsolutePath(path) => format!(r#"sixtyfps::Image::load_from_path(sixtyfps::SharedString(u8"{}"))"#, escape_string(path.as_str())),
-                crate::expression_tree::ImageReference::EmbeddedData(_) => unimplemented!("The C++ generator does not support resource embedding yet")
+                crate::expression_tree::ImageReference::EmbeddedData { .. } => unimplemented!("The C++ generator does not support resource embedding yet")
             }
         }
         Expression::Condition { condition, true_expr, false_expr } => {
@@ -1649,7 +1689,7 @@ fn compile_expression(
             let false_code = return_compile_expression(false_expr, component, Some(&ty));
             format!(
                 r#"[&]() -> {} {{ if ({}) {{ {}; }} else {{ {}; }}}}()"#,
-                ty.cpp_type().unwrap_or("void".into()),
+                ty.cpp_type().unwrap_or_else(|| "void".to_string()),
                 cond_code,
                 true_code,
                 false_code
@@ -1681,7 +1721,7 @@ fn compile_expression(
                         .unwrap_or_else(|| "(Error: missing member in object)".to_owned())
                 });
                 if let Some(name) = name {
-                    format!("{}{{{}}}", name, elem.join(", "))
+                    format!("{}{{{}}}", ident(name), elem.join(", "))
                 } else {
                     format!("std::make_tuple({})", elem.join(", "))
                 }
@@ -1700,15 +1740,15 @@ fn compile_expression(
             let mut stops_it = stops.iter().map(|(color, stop)| {
                 let color = compile_expression(color, component);
                 let position = compile_expression(stop, component);
-                format!("sixtyfps::GradientStop{{ {}, {}, }}", color, position)
+                format!("sixtyfps::private_api::GradientStop{{ {}, {}, }}", color, position)
             });
             format!(
-                "[&] {{ const sixtyfps::GradientStop stops[] = {{ {} }}; return sixtyfps::LinearGradientBrush({}, stops, {}); }}()",
+                "[&] {{ const sixtyfps::private_api::GradientStop stops[] = {{ {} }}; return sixtyfps::Brush(sixtyfps::private_api::LinearGradientBrush({}, stops, {})); }}()",
                 stops_it.join(", "), angle, stops.len()
             )
         }
         Expression::EnumerationValue(value) => {
-            format!("sixtyfps::cbindgen_private::{}::{}", value.enumeration.name, value.to_string())
+            format!("sixtyfps::cbindgen_private::{}::{}", value.enumeration.name, ident(&value.to_string()))
         }
         Expression::ReturnStatement(Some(expr)) => format!(
             "throw sixtyfps::private_api::ReturnWrapper<{}>({})",
@@ -1719,7 +1759,7 @@ fn compile_expression(
         Expression::LayoutCacheAccess { layout_cache_prop, index, repeater_index } => {
             let cache = access_named_reference(layout_cache_prop, component, "self");
             if let Some(ri) = repeater_index {
-                format!("[&](auto cache) {{ return cache[int(cache[{}]) + {} * 2]; }} ({}.get())", index, compile_expression(&ri, component), cache)
+                format!("[&](auto cache) {{ return cache[int(cache[{}]) + {} * 2]; }} ({}.get())", index, compile_expression(ri, component), cache)
             } else {
                 format!("{}.get()[{}]", cache, index)
             }
@@ -1742,7 +1782,7 @@ fn compile_expression(
             let call = if *o == layout.orientation {
                 format!("sixtyfps_box_layout_info(slice, {}, &padding, {})", spacing, alignment)
             } else {
-                format!("sixtyfps_box_layout_info_ortho(slice, &padding)")
+                "sixtyfps_box_layout_info_ortho(slice, &padding)".to_string()
             };
             format!("[&] {{ \
                     const auto padding = {};\
@@ -1845,7 +1885,7 @@ fn compile_assignment(
                         .expect("Expression::ObjectAccess: Cannot find a key in an object");
                     format!("std::get<{}>({})", index, tmpobj)
                 }
-                Type::Struct { .. } => format!("{}.{}", tmpobj, name),
+                Type::Struct { .. } => format!("{}.{}", tmpobj, ident(name)),
                 _ => panic!("Expression::ObjectAccess's base expression is not an Object type"),
             };
             let op = if op == '=' { ' ' } else { op };
@@ -1877,7 +1917,7 @@ fn compile_assignment(
                 "self",
             );
             let index_access = access_member(&parent_component.root_element, "", component, "self");
-            let repeater_id = format!("repeater_{}", element.borrow().id);
+            let repeater_id = format!("repeater_{}", ident(&element.borrow().id));
             if op == '=' {
                 format!(
                     "{}{}.model_set_row_data({}index.get(), {})",
@@ -1958,14 +1998,16 @@ fn box_layout_data(
         let mut repeater_idx = 0usize;
         for item in &layout.elems {
             if item.element.borrow().repeated.is_some() {
-                push_code +=
-                    &format!("self->repeater_{}.ensure_updated(self);", item.element.borrow().id);
+                push_code += &format!(
+                    "self->repeater_{}.ensure_updated(self);",
+                    ident(&item.element.borrow().id)
+                );
                 if repeated_indices.is_some() {
                     push_code += &format!("repeater_indices[{}] = cells.size();", repeater_idx * 2);
                     push_code += &format!(
                         "repeater_indices[{c}] = self->repeater_{id}.inner ? self->repeater_{id}.inner->data.size() : 0;",
                         c = repeater_idx * 2 + 1,
-                        id = item.element.borrow().id
+                        id = ident(&item.element.borrow().id)
                     );
                 }
                 repeater_idx += 1;
@@ -1973,7 +2015,7 @@ fn box_layout_data(
                     "if (self->repeater_{id}.inner) \
                         for (auto &&sub_comp : self->repeater_{id}.inner->data) \
                            cells.push_back((*sub_comp.ptr)->box_layout_data({o}));",
-                    id = item.element.borrow().id,
+                    id = ident(&item.element.borrow().id),
                     o = to_cpp_orientation(orientation),
                 );
             } else {
@@ -2028,10 +2070,10 @@ fn get_layout_info(
         format!("{}.get()", access_named_reference(layout_info_prop, component, "self"))
     } else {
         format!(
-            "{vt}->layouting_info({{{vt}, const_cast<sixtyfps::cbindgen_private::{ty}*>(&self->{id})}}, {o}, &self->window)",
+            "{vt}->layouting_info({{{vt}, const_cast<sixtyfps::cbindgen_private::{ty}*>(&self->{id})}}, {o}, &self->m_window.window_handle())",
             vt = elem.borrow().base_type.as_native().cpp_vtable_getter,
             ty = elem.borrow().base_type.as_native().class_name,
-            id = elem.borrow().id,
+            id = ident(&elem.borrow().id),
             o = to_cpp_orientation(orientation),
         )
     };
@@ -2062,7 +2104,7 @@ fn compile_path(path: &crate::expression_tree::Path, component: &Rc<Component>)
                         .cpp_type
                         .as_ref()
                         .map(|cpp_type| {
-                            new_struct_with_bindings(&cpp_type, &element.bindings, component)
+                            new_struct_with_bindings(cpp_type, &element.bindings, component)
                         })
                         .unwrap_or_default();
                     format!(
@@ -2103,7 +2145,7 @@ fn compile_path(path: &crate::expression_tree::Path, component: &Rc<Component>)
     }
 }
 
-fn compile_path_events(events: &crate::expression_tree::PathEvents) -> (Vec<String>, Vec<String>) {
+fn compile_path_events(events: &[crate::expression_tree::PathEvent]) -> (Vec<String>, Vec<String>) {
     use lyon_path::Event;
 
     let mut coordinates = Vec::new();
