@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -10,7 +10,7 @@ LICENSE END */
 //! Datastructures used to represent layouts in the compiler
 
 use crate::diagnostics::BuildDiagnostics;
-use crate::expression_tree::{Expression, NamedReference, Path};
+use crate::expression_tree::{BindingExpression, Expression, NamedReference, Path};
 use crate::langtype::{PropertyLookupResult, Type};
 use crate::object_tree::ElementRc;
 
@@ -143,41 +143,50 @@ pub struct LayoutConstraints {
 impl LayoutConstraints {
     pub fn new(element: &ElementRc, diag: &mut BuildDiagnostics) -> Self {
         let mut constraints = Self {
-            min_width: binding_reference(&element, "min_width"),
-            max_width: binding_reference(&element, "max_width"),
-            min_height: binding_reference(&element, "min_height"),
-            max_height: binding_reference(&element, "max_height"),
-            preferred_width: binding_reference(&element, "preferred_width"),
-            preferred_height: binding_reference(&element, "preferred_height"),
-            horizontal_stretch: binding_reference(&element, "horizontal_stretch"),
-            vertical_stretch: binding_reference(&element, "vertical_stretch"),
+            min_width: binding_reference(element, "min-width"),
+            max_width: binding_reference(element, "max-width"),
+            min_height: binding_reference(element, "min-height"),
+            max_height: binding_reference(element, "max-height"),
+            preferred_width: binding_reference(element, "preferred-width"),
+            preferred_height: binding_reference(element, "preferred-height"),
+            horizontal_stretch: binding_reference(element, "horizontal-stretch"),
+            vertical_stretch: binding_reference(element, "vertical-stretch"),
             fixed_width: false,
             fixed_height: false,
         };
-        let mut apply_size_constraint = |prop, binding, op: &mut Option<NamedReference>| {
-            if let Some(other_prop) = op {
-                diag.push_error(
-                    format!("Cannot specify both '{}' and '{}'", prop, other_prop.name()),
-                    binding,
-                )
-            }
-            *op = Some(NamedReference::new(element, prop))
-        };
+        let mut apply_size_constraint =
+            |prop, binding: &BindingExpression, op: &mut Option<NamedReference>| {
+                if let Some(other_prop) = op {
+                    let old_priority = other_prop
+                        .element()
+                        .borrow()
+                        .bindings
+                        .get(other_prop.name())
+                        .map_or(0, |b| b.priority);
+                    if old_priority <= binding.priority {
+                        diag.push_error(
+                            format!("Cannot specify both '{}' and '{}'", prop, other_prop.name()),
+                            binding,
+                        )
+                    }
+                }
+                *op = Some(NamedReference::new(element, prop))
+            };
         let e = element.borrow();
-        e.bindings.get("height").map(|s| {
+        if let Some(s) = e.bindings.get("height") {
             constraints.fixed_height = true;
             apply_size_constraint("height", s, &mut constraints.min_height);
             apply_size_constraint("height", s, &mut constraints.max_height);
-        });
-        e.bindings.get("width").map(|s| {
+        }
+        if let Some(s) = e.bindings.get("width") {
             if s.expression.ty() == Type::Percent {
                 apply_size_constraint("width", s, &mut constraints.min_width);
-                return;
+            } else {
+                constraints.fixed_width = true;
+                apply_size_constraint("width", s, &mut constraints.min_width);
+                apply_size_constraint("width", s, &mut constraints.max_width);
             }
-            constraints.fixed_width = true;
-            apply_size_constraint("width", s, &mut constraints.min_width);
-            apply_size_constraint("width", s, &mut constraints.max_width);
-        });
+        }
 
         constraints
     }
@@ -227,14 +236,30 @@ impl LayoutConstraints {
     }
 
     pub fn visit_named_references(&mut self, visitor: &mut impl FnMut(&mut NamedReference)) {
-        self.max_width.as_mut().map(|e| visitor(&mut *e));
-        self.min_width.as_mut().map(|e| visitor(&mut *e));
-        self.max_height.as_mut().map(|e| visitor(&mut *e));
-        self.min_height.as_mut().map(|e| visitor(&mut *e));
-        self.preferred_width.as_mut().map(|e| visitor(&mut *e));
-        self.preferred_height.as_mut().map(|e| visitor(&mut *e));
-        self.horizontal_stretch.as_mut().map(|e| visitor(&mut *e));
-        self.vertical_stretch.as_mut().map(|e| visitor(&mut *e));
+        if let Some(e) = self.max_width.as_mut() {
+            visitor(&mut *e);
+        }
+        if let Some(e) = self.min_width.as_mut() {
+            visitor(&mut *e);
+        }
+        if let Some(e) = self.max_height.as_mut() {
+            visitor(&mut *e);
+        }
+        if let Some(e) = self.min_height.as_mut() {
+            visitor(&mut *e);
+        }
+        if let Some(e) = self.preferred_width.as_mut() {
+            visitor(&mut *e);
+        }
+        if let Some(e) = self.preferred_height.as_mut() {
+            visitor(&mut *e);
+        }
+        if let Some(e) = self.horizontal_stretch.as_mut() {
+            visitor(&mut *e);
+        }
+        if let Some(e) = self.vertical_stretch.as_mut() {
+            visitor(&mut *e);
+        }
     }
 }
 
@@ -267,10 +292,18 @@ pub struct Padding {
 
 impl Padding {
     fn visit_named_references(&mut self, visitor: &mut impl FnMut(&mut NamedReference)) {
-        self.left.as_mut().map(|e| visitor(&mut *e));
-        self.right.as_mut().map(|e| visitor(&mut *e));
-        self.top.as_mut().map(|e| visitor(&mut *e));
-        self.bottom.as_mut().map(|e| visitor(&mut *e));
+        if let Some(e) = self.left.as_mut() {
+            visitor(&mut *e)
+        }
+        if let Some(e) = self.right.as_mut() {
+            visitor(&mut *e)
+        }
+        if let Some(e) = self.top.as_mut() {
+            visitor(&mut *e)
+        }
+        if let Some(e) = self.bottom.as_mut() {
+            visitor(&mut *e)
+        }
     }
 
     // Return reference to the begin and end padding for a given orientation
@@ -293,8 +326,12 @@ pub struct LayoutGeometry {
 impl LayoutGeometry {
     pub fn visit_named_references(&mut self, visitor: &mut impl FnMut(&mut NamedReference)) {
         self.rect.visit_named_references(visitor);
-        self.spacing.as_mut().map(|e| visitor(&mut *e));
-        self.alignment.as_mut().map(|e| visitor(&mut *e));
+        if let Some(e) = self.spacing.as_mut() {
+            visitor(&mut *e)
+        }
+        if let Some(e) = self.alignment.as_mut() {
+            visitor(&mut *e)
+        }
         self.padding.visit_named_references(visitor);
     }
 }
@@ -331,16 +368,16 @@ impl LayoutGeometry {
         let alignment = binding_reference(layout_element, "alignment");
 
         let padding = || binding_reference(layout_element, "padding");
-        init_fake_property(layout_element, "padding_left", padding);
-        init_fake_property(layout_element, "padding_right", padding);
-        init_fake_property(layout_element, "padding_top", padding);
-        init_fake_property(layout_element, "padding_bottom", padding);
+        init_fake_property(layout_element, "padding-left", padding);
+        init_fake_property(layout_element, "padding-right", padding);
+        init_fake_property(layout_element, "padding-top", padding);
+        init_fake_property(layout_element, "padding-bottom", padding);
 
         let padding = Padding {
-            left: binding_reference(layout_element, "padding_left").or_else(padding),
-            right: binding_reference(layout_element, "padding_right").or_else(padding),
-            top: binding_reference(layout_element, "padding_top").or_else(padding),
-            bottom: binding_reference(layout_element, "padding_bottom").or_else(padding),
+            left: binding_reference(layout_element, "padding-left").or_else(padding),
+            right: binding_reference(layout_element, "padding-right").or_else(padding),
+            top: binding_reference(layout_element, "padding-top").or_else(padding),
+            bottom: binding_reference(layout_element, "padding-bottom").or_else(padding),
         };
 
         let rect = LayoutRect::install_on_element(layout_element);
