@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -19,11 +19,14 @@ use crate::langtype::PropertyLookupResult;
 use crate::langtype::{BuiltinElement, NativeClass, Type};
 use crate::layout::{LayoutConstraints, Orientation};
 use crate::namedreference::NamedReference;
-use crate::parser::{identifier_text, syntax_nodes, SyntaxKind, SyntaxNode};
+use crate::parser;
+use crate::parser::{syntax_nodes, SyntaxKind, SyntaxNode};
 use crate::typeloader::ImportedTypes;
 use crate::typeregister::TypeRegister;
 use std::cell::RefCell;
+use std::collections::btree_map::Entry;
 use std::collections::{BTreeMap, HashMap};
+use std::iter::FromIterator;
 use std::rc::{Rc, Weak};
 
 macro_rules! unwrap_or_continue {
@@ -79,7 +82,7 @@ impl Document {
              local_registry: &mut TypeRegister| {
                 let mut ty = type_struct_from_node(n.ObjectType(), diag, local_registry);
                 if let Type::Struct { name, .. } = &mut ty {
-                    *name = identifier_text(&n.DeclaredIdentifier());
+                    *name = parser::identifier_text(&n.DeclaredIdentifier());
                 } else {
                     assert!(diag.has_error());
                     return;
@@ -172,6 +175,18 @@ pub struct PopupWindow {
 
 type ChildrenInsertionPoint = (ElementRc, syntax_nodes::ChildrenPlaceholder);
 
+/// Used sub types for a root component
+#[derive(Debug, Default)]
+pub struct UsedSubTypes {
+    /// All the globals used by the component and its children.
+    pub globals: Vec<Rc<Component>>,
+    /// All the structs used by the component and its children.
+    pub structs: Vec<Type>,
+    /// All the sub components use by this components and its children,
+    /// and the amount of time it is used
+    pub sub_components: Vec<Rc<Component>>,
+}
+
 /// A component is a type in the language which can be instantiated,
 /// Or is materialized for repeated expression.
 #[derive(Default, Debug)]
@@ -202,11 +217,9 @@ pub struct Component {
     /// Code to be inserted into the constructor
     pub setup_code: RefCell<Vec<Expression>>,
 
-    /// All the globals used by this component and its children.
-    /// FIXME: can we have cycle?
-    pub used_global: RefCell<Vec<Rc<Component>>>,
-    pub used_structs: RefCell<Vec<Type>>,
-
+    /// The list of used extra types used (recursively) by this root component.
+    /// (This only make sense on the root component)
+    pub used_types: RefCell<UsedSubTypes>,
     pub popup_windows: RefCell<Vec<PopupWindow>>,
 }
 
@@ -218,7 +231,7 @@ impl Component {
     ) -> Rc<Self> {
         let mut child_insertion_point = None;
         let c = Component {
-            id: identifier_text(&node.DeclaredIdentifier()).unwrap_or_default(),
+            id: parser::identifier_text(&node.DeclaredIdentifier()).unwrap_or_default(),
             root_element: Element::from_node(
                 node.Element(),
                 "root".into(),
@@ -293,7 +306,7 @@ impl TransitionPropertyAnimation {
         Expression::BinaryExpression {
             lhs: Box::new(Expression::StructFieldAccess {
                 base: Box::new(state),
-                name: (if self.is_out { "previous_state" } else { "current_state" }).into(),
+                name: (if self.is_out { "previous-state" } else { "current-state" }).into(),
             }),
             rhs: Box::new(Expression::NumberLiteral(self.state_id as _, Unit::None)),
             op: '=',
@@ -301,12 +314,109 @@ impl TransitionPropertyAnimation {
     }
 }
 
-#[derive(Debug, Clone)]
+#[derive(Debug)]
 pub enum PropertyAnimation {
     Static(ElementRc),
     Transition { state_ref: Expression, animations: Vec<TransitionPropertyAnimation> },
 }
 
+impl Clone for PropertyAnimation {
+    fn clone(&self) -> Self {
+        fn deep_clone(e: &ElementRc) -> ElementRc {
+            let e = e.borrow();
+            debug_assert!(e.children.is_empty());
+            debug_assert!(e.property_declarations.is_empty());
+            debug_assert!(e.states.is_empty() && e.transitions.is_empty());
+            Rc::new(RefCell::new(Element {
+                id: e.id.clone(),
+                base_type: e.base_type.clone(),
+                bindings: e.bindings.clone(),
+                property_analysis: e.property_analysis.clone(),
+                enclosing_component: e.enclosing_component.clone(),
+                repeated: None,
+                node: e.node.clone(),
+                ..Default::default()
+            }))
+        }
+        match self {
+            PropertyAnimation::Static(e) => PropertyAnimation::Static(deep_clone(e)),
+            PropertyAnimation::Transition { state_ref, animations } => {
+                PropertyAnimation::Transition {
+                    state_ref: state_ref.clone(),
+                    animations: animations
+                        .iter()
+                        .map(|t| TransitionPropertyAnimation {
+                            state_id: t.state_id,
+                            is_out: t.is_out,
+                            animation: deep_clone(&t.animation),
+                        })
+                        .collect(),
+                }
+            }
+        }
+    }
+}
+
+// Wrapper around BindingsMap to offer an animation-preserving "insert" variant.
+#[derive(Debug, Clone, Default, derive_more::Deref, derive_more::DerefMut)]
+pub struct BindingsMap(BTreeMap<String, BindingExpression>);
+
+impl BindingsMap {
+    pub fn set_binding_if_not_set(
+        &mut self,
+        property_name: String,
+        expression_fn: impl FnOnce() -> BindingExpression,
+    ) {
+        match self.0.entry(property_name) {
+            Entry::Vacant(vacant_entry) => {
+                vacant_entry.insert(expression_fn());
+            }
+            Entry::Occupied(mut existing_entry)
+                if matches!(existing_entry.get().expression, Expression::Invalid) =>
+            {
+                existing_entry.get_mut().merge_with(&expression_fn());
+            }
+            _ => {}
+        };
+    }
+}
+
+impl IntoIterator for BindingsMap {
+    type Item = (String, BindingExpression);
+
+    type IntoIter = std::collections::btree_map::IntoIter<String, BindingExpression>;
+
+    fn into_iter(self) -> Self::IntoIter {
+        self.0.into_iter()
+    }
+}
+
+impl<'a> IntoIterator for &'a BindingsMap {
+    type Item = (&'a String, &'a BindingExpression);
+
+    type IntoIter = std::collections::btree_map::Iter<'a, String, BindingExpression>;
+
+    fn into_iter(self) -> Self::IntoIter {
+        (&self.0).iter()
+    }
+}
+
+impl<'a> IntoIterator for &'a mut BindingsMap {
+    type Item = (&'a String, &'a mut BindingExpression);
+
+    type IntoIter = std::collections::btree_map::IterMut<'a, String, BindingExpression>;
+
+    fn into_iter(self) -> Self::IntoIter {
+        self.0.iter_mut()
+    }
+}
+
+impl FromIterator<(String, BindingExpression)> for BindingsMap {
+    fn from_iter<T: IntoIterator<Item = (String, BindingExpression)>>(iter: T) -> Self {
+        Self(BTreeMap::from_iter(iter))
+    }
+}
+
 /// An Element is an instantiation of a Component
 #[derive(Default)]
 pub struct Element {
@@ -319,7 +429,7 @@ pub struct Element {
     //pub base: QualifiedTypeName,
     pub base_type: crate::langtype::Type,
     /// Currently contains also the callbacks. FIXME: should that be changed?
-    pub bindings: BTreeMap<String, BindingExpression>,
+    pub bindings: BindingsMap,
     pub property_analysis: RefCell<HashMap<String, PropertyAnalysis>>,
 
     pub children: Vec<ElementRc>,
@@ -331,8 +441,6 @@ pub struct Element {
     /// Main owner for a reference to a property.
     pub named_references: crate::namedreference::NamedReferenceContainer,
 
-    pub property_animations: HashMap<String, PropertyAnimation>,
-
     /// Tis element is part of a `for <xxx> in <model>:
     pub repeated: Option<RepeatedElementInfo>,
 
@@ -402,12 +510,15 @@ pub fn pretty_print(
         indent!();
         write!(f, "{}: ", name)?;
         expression_tree::pretty_print(f, &expr.expression)?;
+        if expr.analysis.borrow().as_ref().map_or(false, |a| a.is_const) {
+            write!(f, "/*const*/")?;
+        }
         writeln!(f, ";")?;
         //writeln!(f, "; /*{}*/", expr.priority)?;
-    }
-    for (name, anim) in &e.property_animations {
-        indent!();
-        writeln!(f, "animate {} {:?}", name, anim)?;
+        if let Some(anim) = &expr.animation {
+            indent!();
+            writeln!(f, "animate {} {:?}", name, anim)?;
+        }
     }
     if !e.states.is_empty() {
         indent!();
@@ -452,7 +563,9 @@ pub struct ListViewInfo {
     pub viewport_y: NamedReference,
     pub viewport_height: NamedReference,
     pub viewport_width: NamedReference,
+    /// The ListView's inner visible height (not counting eventual scrollbar)
     pub listview_height: NamedReference,
+    /// The ListView's inner visible width (not counting eventual scrollbar)
     pub listview_width: NamedReference,
 }
 
@@ -484,6 +597,9 @@ impl Element {
         let base_type = if let Some(base_node) = node.QualifiedName() {
             let base = QualifiedTypeName::from_node(base_node.clone());
             let base_string = base.to_string();
+            if base_string == "Clip" {
+                diag.push_warning("The 'Clip' element is deprecated, use the 'clip' property on a Rectangle instead".into(), &base_node);
+            }
             match parent_type.lookup_type_for_child_element(&base_string, tr) {
                 Ok(Type::Component(c)) if c.is_global() => {
                     diag.push_error(
@@ -515,7 +631,9 @@ impl Element {
             };
             node.SubElement().for_each(|n| error_on(&n, "sub elements"));
             node.RepeatedElement().for_each(|n| error_on(&n, "sub elements"));
-            node.ChildrenPlaceholder().map(|n| error_on(&n, "sub elements"));
+            if let Some(n) = node.ChildrenPlaceholder() {
+                error_on(&n, "sub elements");
+            }
             node.CallbackConnection().for_each(|n| error_on(&n, "callback connections"));
             node.PropertyAnimation().for_each(|n| error_on(&n, "animations"));
             node.States().for_each(|n| error_on(&n, "states"));
@@ -542,7 +660,7 @@ impl Element {
                 .unwrap_or(Type::InferredProperty);
 
             let unresolved_prop_name =
-                unwrap_or_continue!(identifier_text(&prop_decl.DeclaredIdentifier()); diag);
+                unwrap_or_continue!(parser::identifier_text(&prop_decl.DeclaredIdentifier()); diag);
             let PropertyLookupResult {
                 resolved_name: prop_name,
                 property_type: maybe_existing_prop_type,
@@ -608,7 +726,8 @@ impl Element {
         }
 
         for sig_decl in node.CallbackDeclaration() {
-            let name = unwrap_or_continue!(identifier_text(&sig_decl.DeclaredIdentifier()); diag);
+            let name =
+                unwrap_or_continue!(parser::identifier_text(&sig_decl.DeclaredIdentifier()); diag);
 
             if let Some(csn) = sig_decl.TwoWayBinding() {
                 r.bindings.insert(name.clone(), BindingExpression::new_uncompiled(csn.into()));
@@ -638,7 +757,7 @@ impl Element {
         }
 
         for con_node in node.CallbackConnection() {
-            let unresolved_name = unwrap_or_continue!(identifier_text(&con_node); diag);
+            let unresolved_name = unwrap_or_continue!(parser::identifier_text(&con_node); diag);
             let PropertyLookupResult { resolved_name, property_type } =
                 r.lookup_property(&unresolved_name);
             if let Type::Callback { args, .. } = &property_type {
@@ -688,7 +807,7 @@ impl Element {
                 match QualifiedTypeName::from_node(prop_name_token.clone()).members.as_slice() {
                     [unresolved_prop_name] => {
                         let PropertyLookupResult { resolved_name, property_type } =
-                            r.lookup_property(&unresolved_prop_name);
+                            r.lookup_property(unresolved_prop_name);
                         if let Some(anim_element) = animation_element_from_node(
                             &anim,
                             &prop_name_token,
@@ -698,17 +817,21 @@ impl Element {
                         ) {
                             if unresolved_prop_name != resolved_name.as_ref() {
                                 diag.push_property_deprecation_warning(
-                                    &unresolved_prop_name,
+                                    unresolved_prop_name,
                                     &resolved_name,
                                     &prop_name_token,
                                 );
                             }
 
-                            if r.property_animations
-                                .insert(
-                                    resolved_name.to_string(),
-                                    PropertyAnimation::Static(anim_element),
-                                )
+                            let expr_binding =
+                                r.bindings.entry(resolved_name.to_string()).or_insert_with(|| {
+                                    let mut r = BindingExpression::from(Expression::Invalid);
+                                    r.priority = 1;
+                                    r
+                                });
+                            if expr_binding
+                                .animation
+                                .replace(PropertyAnimation::Static(anim_element))
                                 .is_some()
                             {
                                 diag.push_error("Duplicated animation".into(), &prop_name_token)
@@ -779,13 +902,13 @@ impl Element {
 
         for state in node.States().flat_map(|s| s.State()) {
             let s = State {
-                id: identifier_text(&state.DeclaredIdentifier()).unwrap_or_default(),
+                id: parser::identifier_text(&state.DeclaredIdentifier()).unwrap_or_default(),
                 condition: state.Expression().map(|e| Expression::Uncompiled(e.into())),
                 property_changes: state
                     .StatePropertyChange()
                     .filter_map(|s| {
                         lookup_property_from_qualified_name(s.QualifiedName(), &r, diag).map(
-                            |(ne, _)| (ne, Expression::Uncompiled(s.BindingExpression().into())),
+                            |(ne, _)| (ne, Expression::Uncompiled(s.BindingExpression().into()), s),
                         )
                     })
                     .collect(),
@@ -798,8 +921,8 @@ impl Element {
                 diag.push_error("TODO: catch-all not yet implemented".into(), &star);
             };
             let trans = Transition {
-                is_out: identifier_text(&trs).unwrap_or_default() == "out",
-                state_id: identifier_text(&trs.DeclaredIdentifier()).unwrap_or_default(),
+                is_out: parser::identifier_text(&trs).unwrap_or_default() == "out",
+                state_id: parser::identifier_text(&trs.DeclaredIdentifier()).unwrap_or_default(),
                 property_animations: trs
                     .PropertyAnimation()
                     .flat_map(|pa| pa.QualifiedName().map(move |qn| (pa.clone(), qn)))
@@ -827,7 +950,7 @@ impl Element {
         diag: &mut BuildDiagnostics,
         tr: &TypeRegister,
     ) -> ElementRc {
-        let id = identifier_text(&node).unwrap_or_default();
+        let id = parser::identifier_text(&node).unwrap_or_default();
         if matches!(id.as_ref(), "parent" | "self" | "root") {
             diag.push_error(
                 format!("'{}' is a reserved id", id),
@@ -853,11 +976,11 @@ impl Element {
     ) -> ElementRc {
         let is_listview = if parent.borrow().base_type.to_string() == "ListView" {
             Some(ListViewInfo {
-                viewport_y: NamedReference::new(parent, "viewport_y"),
-                viewport_height: NamedReference::new(parent, "viewport_height"),
-                viewport_width: NamedReference::new(parent, "viewport_width"),
-                listview_height: NamedReference::new(parent, "visible_height"),
-                listview_width: NamedReference::new(parent, "visible_width"),
+                viewport_y: NamedReference::new(parent, "viewport-y"),
+                viewport_height: NamedReference::new(parent, "viewport-height"),
+                viewport_width: NamedReference::new(parent, "viewport-width"),
+                listview_height: NamedReference::new(parent, "visible-height"),
+                listview_width: NamedReference::new(parent, "visible-width"),
             })
         } else {
             None
@@ -866,9 +989,12 @@ impl Element {
             model: Expression::Uncompiled(node.Expression().into()),
             model_data_id: node
                 .DeclaredIdentifier()
-                .and_then(|n| identifier_text(&n))
+                .and_then(|n| parser::identifier_text(&n))
+                .unwrap_or_default(),
+            index_id: node
+                .RepeatedIndex()
+                .and_then(|r| parser::identifier_text(&r))
                 .unwrap_or_default(),
-            index_id: node.RepeatedIndex().and_then(|r| identifier_text(&r)).unwrap_or_default(),
             is_conditional_element: false,
             is_listview,
         };
@@ -1047,7 +1173,10 @@ pub fn type_struct_from_node(
     let fields = object_node
         .ObjectTypeMember()
         .map(|member| {
-            (identifier_text(&member).unwrap_or_default(), type_from_node(member.Type(), diag, tr))
+            (
+                parser::identifier_text(&member).unwrap_or_default(),
+                type_from_node(member.Type(), diag, tr),
+            )
         })
         .collect();
     Type::Struct { fields, name: None, node: Some(object_node) }
@@ -1120,10 +1249,10 @@ fn lookup_property_from_qualified_name(
             if !property_type.is_property_type() {
                 diag.push_error(format!("'{}' is not a valid property", qualname), &node);
             }
-            Some((NamedReference::new(&r, &resolved_name), property_type))
+            Some((NamedReference::new(r, &resolved_name), property_type))
         }
         [elem_id, unresolved_prop_name] => {
-            if let Some(element) = find_element_by_id(&r, elem_id.as_ref()) {
+            if let Some(element) = find_element_by_id(r, elem_id.as_ref()) {
                 let PropertyLookupResult { resolved_name, property_type } =
                     element.borrow().lookup_property(unresolved_prop_name.as_ref());
                 if !property_type.is_property_type() {
@@ -1280,6 +1409,17 @@ pub fn visit_element_expressions(
         let mut bindings = std::mem::take(&mut elem.borrow_mut().bindings);
         for (name, expr) in &mut bindings {
             vis(expr, Some(name.as_str()), &|| elem.borrow().lookup_property(name).property_type);
+
+            match &mut expr.animation {
+                Some(PropertyAnimation::Static(e)) => visit_element_expressions_simple(e, vis),
+                Some(PropertyAnimation::Transition { animations, state_ref }) => {
+                    vis(state_ref, None, &|| Type::Int32);
+                    for a in animations {
+                        visit_element_expressions_simple(&a.animation, vis)
+                    }
+                }
+                None => (),
+            }
         }
         elem.borrow_mut().bindings = bindings;
     }
@@ -1296,7 +1436,7 @@ pub fn visit_element_expressions(
         if let Some(cond) = s.condition.as_mut() {
             vis(cond, None, &|| Type::Bool)
         }
-        for (ne, e) in &mut s.property_changes {
+        for (ne, e, _) in &mut s.property_changes {
             vis(e, Some(ne.name()), &|| {
                 ne.element().borrow().lookup_property(ne.name()).property_type
             });
@@ -1311,20 +1451,6 @@ pub fn visit_element_expressions(
         }
     }
     elem.borrow_mut().transitions = transitions;
-
-    let mut property_animations = std::mem::take(&mut elem.borrow_mut().property_animations);
-    for anim_elem in property_animations.values_mut() {
-        match anim_elem {
-            PropertyAnimation::Static(e) => visit_element_expressions_simple(e, &mut vis),
-            PropertyAnimation::Transition { animations, state_ref } => {
-                vis(state_ref, None, &|| Type::Int32);
-                for a in animations {
-                    visit_element_expressions_simple(&a.animation, &mut vis)
-                }
-            }
-        }
-    }
-    elem.borrow_mut().property_animations = property_animations;
 }
 
 /// Visit all the named reference in an element
@@ -1357,7 +1483,7 @@ pub fn visit_all_named_references_in_element(
     visit_element_expressions(elem, |expr, _, _| recurse_expression(expr, &mut vis));
     let mut states = std::mem::take(&mut elem.borrow_mut().states);
     for s in &mut states {
-        for (r, _) in &mut s.property_changes {
+        for (r, _, _) in &mut s.property_changes {
             vis(r);
         }
     }
@@ -1385,7 +1511,7 @@ pub fn visit_all_named_references_in_element(
     elem.borrow_mut().layout_info_prop = layout_info_prop;
 
     let mut property_declarations = std::mem::take(&mut elem.borrow_mut().property_declarations);
-    for (_, pd) in &mut property_declarations {
+    for pd in property_declarations.values_mut() {
         pd.is_alias.as_mut().map(&mut vis);
     }
     elem.borrow_mut().property_declarations = property_declarations;
@@ -1431,7 +1557,7 @@ pub fn visit_all_expressions(
 pub struct State {
     pub id: String,
     pub condition: Option<Expression>,
-    pub property_changes: Vec<(NamedReference, Expression)>,
+    pub property_changes: Vec<(NamedReference, Expression, syntax_nodes::StatePropertyChange)>,
 }
 
 #[derive(Debug, Clone)]
@@ -1465,14 +1591,14 @@ impl Exports {
             .ExportsList()
             .flat_map(|exports| exports.ExportSpecifier())
             .map(|export_specifier| {
-                let internal_name = identifier_text(&export_specifier.ExportIdentifier())
+                let internal_name = parser::identifier_text(&export_specifier.ExportIdentifier())
                     .unwrap_or_else(|| {
                         debug_assert!(diag.has_error());
                         String::new()
                     });
                 let exported_name = export_specifier
                     .ExportName()
-                    .and_then(|ident| identifier_text(&ident))
+                    .and_then(|ident| parser::identifier_text(&ident))
                     .unwrap_or_else(|| internal_name.clone());
                 NamedExport {
                     internal_name_ident: export_specifier.ExportIdentifier().into(),
@@ -1484,10 +1610,11 @@ impl Exports {
 
         exports.extend(doc.ExportsList().filter_map(|exports| exports.Component()).map(
             |component| {
-                let name = identifier_text(&component.DeclaredIdentifier()).unwrap_or_else(|| {
-                    debug_assert!(diag.has_error());
-                    String::new()
-                });
+                let name =
+                    parser::identifier_text(&component.DeclaredIdentifier()).unwrap_or_else(|| {
+                        debug_assert!(diag.has_error());
+                        String::new()
+                    });
                 NamedExport {
                     internal_name_ident: component.DeclaredIdentifier().into(),
                     internal_name: name.clone(),
@@ -1497,7 +1624,7 @@ impl Exports {
         ));
         exports.extend(doc.ExportsList().flat_map(|exports| exports.StructDeclaration()).map(
             |st| {
-                let name = identifier_text(&st.DeclaredIdentifier()).unwrap_or_else(|| {
+                let name = parser::identifier_text(&st.DeclaredIdentifier()).unwrap_or_else(|| {
                     debug_assert!(diag.has_error());
                     String::new()
                 });
@@ -1568,7 +1695,7 @@ pub fn inject_element_as_repeated_element(repeated_element: &ElementRc, new_root
 
     // Any elements with a weak reference to the repeater's component will need fixing later.
     let mut elements_with_enclosing_component_reference = Vec::new();
-    recurse_elem(&old_root, &(), &mut |element: &ElementRc, _| {
+    recurse_elem(old_root, &(), &mut |element: &ElementRc, _| {
         if let Some(enclosing_component) = element.borrow().enclosing_component.upgrade() {
             if Rc::ptr_eq(&enclosing_component, &component) {
                 elements_with_enclosing_component_reference.push(element.clone());
@@ -1581,6 +1708,11 @@ pub fn inject_element_as_repeated_element(repeated_element: &ElementRc, new_root
 
     new_root.borrow_mut().child_of_layout =
         std::mem::replace(&mut old_root.borrow_mut().child_of_layout, false);
+    new_root.borrow_mut().layout_info_prop = old_root.borrow().layout_info_prop.clone();
+    new_root
+        .borrow_mut()
+        .bindings
+        .extend(["x", "y"].iter().filter_map(|x| old_root.borrow_mut().bindings.remove_entry(*x)));
 
     // Replace the repeated component's element with our shadow element. That requires a bit of reference counting
     // surgery and relies on nobody having a strong reference left to the component, which we take out of the Rc.
