@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -330,7 +330,7 @@ declare_syntax! {
         /// `-> type`  (but without the ->)
         ReturnType -> [Type],
         CallbackConnection -> [ *DeclaredIdentifier,  CodeBlock ],
-        /// Declaration of a propery.
+        /// Declaration of a property.
         PropertyDeclaration-> [ ?Type , DeclaredIdentifier, ?BindingExpression, ?TwoWayBinding ],
         /// QualifiedName are the properties name
         PropertyAnimation-> [ *QualifiedName, *Binding ],
@@ -351,7 +351,7 @@ declare_syntax! {
         Expression-> [ ?Expression, ?FunctionCallExpression, ?SelfAssignment,
                        ?ConditionalExpression, ?QualifiedName, ?BinaryExpression, ?Array, ?ObjectLiteral,
                        ?UnaryOpExpression, ?CodeBlock, ?StringTemplate, ?AtImageUrl, ?AtLinearGradient],
-        /// Concetenate the Expressions to make a string (usually expended from a template string)
+        /// Concatenate the Expressions to make a string (usually expended from a template string)
         StringTemplate -> [*Expression],
         /// `@image-url("foo.png")`
         AtImageUrl -> [],
@@ -381,7 +381,7 @@ declare_syntax! {
         StatePropertyChange -> [ QualifiedName, BindingExpression ],
         /// `transitions: [...]`
         Transitions -> [*Transition],
-        /// There is an idientfier "in" or "out", the DeclaredIdentifier is the state name
+        /// There is an identifier "in" or "out", the DeclaredIdentifier is the state name
         Transition -> [DeclaredIdentifier, *PropertyAnimation],
         /// Export a set of declared components by name
         ExportsList -> [ *ExportSpecifier, ?Component, *StructDeclaration ],
@@ -487,7 +487,7 @@ mod parser_trait {
         fn peek(&mut self) -> Token {
             self.nth(0)
         }
-        /// Peek the n'th token, not including whitespace and comments
+        /// Peek the `n`th token, not including whitespace and comments
         fn nth(&mut self, n: usize) -> Token;
         fn consume(&mut self);
         fn error(&mut self, e: impl Into<String>);
@@ -569,7 +569,7 @@ impl<'a> DefaultParser<'a> {
 
     /// Constructor that create a parser from the source code
     pub fn new(source: &str, diags: &'a mut BuildDiagnostics) -> Self {
-        Self::from_tokens(crate::lexer::lex(&source), diags)
+        Self::from_tokens(crate::lexer::lex(source), diags)
     }
 
     fn current_token(&self) -> Token {
@@ -601,7 +601,7 @@ impl Parser for DefaultParser<'_> {
         self.builder.finish_node();
     }
 
-    /// Peek the n'th token, not including whitespace and comments
+    /// Peek the `n`th token, not including whitespace and comments
     fn nth(&mut self, mut n: usize) -> Token {
         self.consume_ws();
         let mut c = self.cursor;
@@ -749,7 +749,7 @@ impl NodeOrToken {
 
     pub fn as_node(&self) -> Option<&SyntaxNode> {
         match self {
-            NodeOrToken::Node(n) => Some(&n),
+            NodeOrToken::Node(n) => Some(n),
             NodeOrToken::Token(_) => None,
         }
     }
@@ -757,7 +757,7 @@ impl NodeOrToken {
     pub fn as_token(&self) -> Option<&SyntaxToken> {
         match self {
             NodeOrToken::Node(_) => None,
-            NodeOrToken::Token(t) => Some(&t),
+            NodeOrToken::Token(t) => Some(t),
         }
     }
 
@@ -846,7 +846,7 @@ pub fn identifier_text(node: &SyntaxNode) -> Option<String> {
 }
 
 pub fn normalize_identifier(ident: &str) -> String {
-    ident.replace('-', "_")
+    ident.replace('_', "-")
 }
 
 // Actual parser
