@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -100,8 +100,8 @@ pub fn parse_element_content(p: &mut impl Parser) {
                     }
                     p.consume()
                 } else {
-                    p.consume();
-                    p.error("Unexpected identifier. Expected @children.")
+                    p.test(SyntaxKind::Identifier);
+                    p.error("Parse error: Expected @children")
                 }
             }
             _ => {
@@ -214,13 +214,11 @@ fn parse_binding_expression(p: &mut impl Parser) -> bool {
         parse_code_block(&mut *p);
         p.test(SyntaxKind::Semicolon);
         true
+    } else if parse_expression(&mut *p) {
+        p.expect(SyntaxKind::Semicolon)
     } else {
-        if parse_expression(&mut *p) {
-            p.expect(SyntaxKind::Semicolon)
-        } else {
-            p.test(SyntaxKind::Semicolon);
-            false
-        }
+        p.test(SyntaxKind::Semicolon);
+        false
     }
 }
 
@@ -352,7 +350,7 @@ fn parse_property_declaration(p: &mut impl Parser) {
     } else if p.nth(0).kind() == SyntaxKind::Identifier
         && p.nth(1).kind() != SyntaxKind::DoubleArrow
     {
-        p.error("Missing type. The syntax to declare a property is `property <type> name;`. Only two way bindings can omit the type.");
+        p.error("Missing type. The syntax to declare a property is `property <type> name;`. Only two way bindings can omit the type");
     }
 
     {
