@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -37,7 +37,7 @@ pub fn parse_statement(p: &mut impl Parser) -> bool {
     }
 
     if p.peek().as_str() == "return" {
-        let mut p = p.start_node_at(checkpoint.clone(), SyntaxKind::ReturnStatement);
+        let mut p = p.start_node_at(checkpoint, SyntaxKind::ReturnStatement);
         p.expect(SyntaxKind::Identifier); // "return"
         if !p.test(SyntaxKind::Semicolon) {
             parse_expression(&mut *p);
