@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -18,9 +18,9 @@ use crate::langtype::Type;
 use crate::object_tree::Component;
 
 /// Ideally we would be able to write this in builtin.60,  but the StyleMetrics is not available there
-pub async fn apply_default_properties_from_style<'a>(
+pub async fn apply_default_properties_from_style(
     root_component: &std::rc::Rc<Component>,
-    type_loader: &mut crate::typeloader::TypeLoader<'a>,
+    type_loader: &mut crate::typeloader::TypeLoader<'_>,
     _diag: &mut BuildDiagnostics,
 ) {
     // Ignore import errors
@@ -31,25 +31,25 @@ pub async fn apply_default_properties_from_style<'a>(
     let style_metrics = if let Some(Type::Component(c)) = style_metrics { c } else { return };
 
     crate::object_tree::recurse_elem_including_sub_components(
-        &root_component,
+        root_component,
         &(),
         &mut |elem, _| {
             let mut elem = elem.borrow_mut();
-            match elem.native_class().as_ref().map(|nc| nc.class_name.as_str()) {
-                Some("TextInput") => {
-                    elem.bindings.entry("text_cursor_width".into()).or_insert_with(|| {
+            match elem.base_type.to_string().as_str() {
+                "TextInput" => {
+                    elem.bindings.set_binding_if_not_set("text-cursor-width".into(), || {
                         Expression::PropertyReference(NamedReference::new(
                             &style_metrics.root_element,
-                            "text_cursor_width",
+                            "text-cursor-width",
                         ))
                         .into()
                     });
                 }
-                Some("Window") => {
-                    elem.bindings.entry("background".into()).or_insert_with(|| {
+                "Window" => {
+                    elem.bindings.set_binding_if_not_set("background".into(), || {
                         Expression::PropertyReference(NamedReference::new(
                             &style_metrics.root_element,
-                            "window_background",
+                            "window-background",
                         ))
                         .into()
                     });
