@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -12,7 +12,7 @@ LICENSE END */
 use std::cell::RefCell;
 use std::rc::Rc;
 
-use crate::diagnostics::BuildDiagnostics;
+use crate::diagnostics::{BuildDiagnostics, Spanned};
 use crate::expression_tree::{BindingExpression, Expression, NamedReference};
 use crate::langtype::{NativeClass, Type};
 use crate::object_tree::{Component, Element, ElementRc};
@@ -26,50 +26,41 @@ pub fn handle_clip(
     let native_clip = type_register.lookup("Clip").as_builtin().native_class.clone();
 
     crate::object_tree::recurse_elem_including_sub_components(
-        &component,
+        component,
         &(),
         &mut |elem_rc: &ElementRc, _| {
-            let mut elem = elem_rc.borrow_mut();
-            if let Some(clip_prop) = elem.bindings.remove("clip") {
+            let elem = elem_rc.borrow();
+            if elem.native_class().map_or(false, |n| Rc::ptr_eq(&n, &native_clip)) {
+                return;
+            }
+            if elem.bindings.contains_key("clip")
+                || elem.property_analysis.borrow().get("clip").map_or(false, |a| a.is_set)
+            {
                 match elem.builtin_type().as_ref().map(|ty| ty.name.as_str()) {
                     Some("Rectangle") => {}
                     Some("Path") => {
-                        // it's an actual property, so keep the binding
-                        elem.bindings.insert("clip".into(), clip_prop);
+                        // it's an actual property, so keep it as is
                         return;
                     }
                     _ => {
                         diag.push_error(
-                            "The 'clip' property can only be applied to a Rectangle or a Path for now"
-                                .into(),
-                            &clip_prop.span,
+                            "The 'clip' property can only be applied to a Rectangle or a Path for now".into(),
+                            &elem.bindings.get("clip").and_then(|x| x.span.clone()).or_else(|| elem.node.as_ref().map(|e| e.to_source_location())),
                         );
                         return;
                     }
                 }
-                // Was added by the materialize_fake_properties pass
-                elem.property_declarations.remove("clip");
-                match &clip_prop.expression {
-                    Expression::BoolLiteral(false) => {}
-                    Expression::BoolLiteral(true) => {
-                        drop(elem);
-                        create_clip_element(elem_rc, &native_clip);
-                    }
-                    _ => diag.push_error(
-                        "The 'clip' property can only be a boolean literal (true or false) for now"
-                            .into(),
-                        &clip_prop.span,
-                    ),
-                }
+                drop(elem);
+                create_clip_element(elem_rc, &native_clip);
             }
         },
-    )
+    );
 }
 
 fn create_clip_element(parent_elem: &ElementRc, native_clip: &Rc<NativeClass>) {
     let mut parent = parent_elem.borrow_mut();
     let clip = Rc::new(RefCell::new(Element {
-        id: format!("{}_clip", parent.id),
+        id: format!("{}-clip", parent.id),
         base_type: Type::Native(native_clip.clone()),
         children: std::mem::take(&mut parent.children),
         enclosing_component: parent.enclosing_component.clone(),
@@ -87,7 +78,7 @@ fn create_clip_element(parent_elem: &ElementRc, native_clip: &Rc<NativeClass>) {
             )
         })
         .collect();
-    for optional_binding in ["border_radius", "border_width"].iter() {
+    for optional_binding in ["border-radius", "border-width"].iter() {
         if parent_elem.borrow().bindings.contains_key(*optional_binding) {
             clip.borrow_mut().bindings.insert(
                 optional_binding.to_string(),
@@ -96,4 +87,8 @@ fn create_clip_element(parent_elem: &ElementRc, native_clip: &Rc<NativeClass>) {
             );
         }
     }
+    clip.borrow_mut().bindings.insert(
+        "clip".to_owned(),
+        Expression::TwoWayBinding(NamedReference::new(parent_elem, "clip"), None).into(),
+    );
 }
