@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -32,7 +32,7 @@ use crate::{
 
 pub fn default_geometry(root_component: &Rc<Component>, diag: &mut BuildDiagnostics) {
     crate::object_tree::recurse_elem_including_sub_components(
-        &root_component,
+        root_component,
         &None,
         &mut |elem: &ElementRc, parent: &Option<ElementRc>| {
             fix_percent_size(elem, parent, "width", diag);
@@ -90,12 +90,11 @@ pub fn default_geometry(root_component: &Rc<Component>, diag: &mut BuildDiagnost
                                 let PropertyLookupResult {
                                     resolved_name: image_fit_prop_name,
                                     property_type: image_fit_prop_type,
-                                } = elem.borrow().lookup_property("image_fit");
+                                } = elem.borrow().lookup_property("image-fit");
 
-                                elem.borrow_mut()
-                                    .bindings
-                                    .entry(image_fit_prop_name.into())
-                                    .or_insert_with(|| {
+                                elem.borrow_mut().bindings.set_binding_if_not_set(
+                                    image_fit_prop_name.into(),
+                                    || {
                                         Expression::EnumerationValue(
                                             image_fit_prop_type
                                                 .as_enum()
@@ -104,7 +103,8 @@ pub fn default_geometry(root_component: &Rc<Component>, diag: &mut BuildDiagnost
                                                 .unwrap(),
                                         )
                                         .into()
-                                    });
+                                    },
+                                );
                             }
                         }
                     }
@@ -125,7 +125,13 @@ fn gen_layout_info_prop(elem: &ElementRc) {
         .borrow()
         .children
         .iter()
-        .filter_map(|c| c.borrow().layout_info_prop.clone())
+        .filter(|c| {
+            !c.borrow().bindings.contains_key("x") && !c.borrow().bindings.contains_key("y")
+        })
+        .filter_map(|c| {
+            gen_layout_info_prop(c);
+            c.borrow().layout_info_prop.clone()
+        })
         .collect::<Vec<_>>();
 
     if child_infos.is_empty() {
@@ -134,12 +140,12 @@ fn gen_layout_info_prop(elem: &ElementRc) {
 
     let li_v = super::lower_layout::create_new_prop(
         elem,
-        "layoutinfo_v",
+        "layoutinfo-v",
         crate::layout::layout_info_type(),
     );
     let li_h = super::lower_layout::create_new_prop(
         elem,
-        "layoutinfo_h",
+        "layoutinfo-h",
         crate::layout::layout_info_type(),
     );
     elem.borrow_mut().layout_info_prop = Some((li_h.clone(), li_v.clone()));
@@ -205,14 +211,14 @@ fn make_default_100(elem: &ElementRc, parent_element: &ElementRc, property: &str
     if property_type != Type::LogicalLength {
         return;
     }
-    elem.borrow_mut().bindings.entry(resolved_name.to_string()).or_insert_with(|| {
+    elem.borrow_mut().bindings.set_binding_if_not_set(resolved_name.to_string(), || {
         Expression::PropertyReference(NamedReference::new(parent_element, resolved_name.as_ref()))
             .into()
     });
 }
 
 fn make_default_implicit(elem: &ElementRc, property: &str, orientation: Orientation) {
-    elem.borrow_mut().bindings.entry(property.into()).or_insert_with(|| {
+    elem.borrow_mut().bindings.set_binding_if_not_set(property.into(), || {
         Expression::StructFieldAccess {
             base: implicit_layout_info_call(elem, orientation).into(),
             name: "preferred".into(),
