@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -27,17 +27,17 @@ use crate::langtype::{NativeClass, Type};
 use crate::object_tree::{Component, Element, ElementRc};
 use crate::typeregister::TypeRegister;
 
-pub fn handle_flickable(root_component: &Rc<Component>, tr: &TypeRegister) -> () {
+pub fn handle_flickable(root_component: &Rc<Component>, tr: &TypeRegister) {
     let mut native_rect = tr.lookup("Rectangle").as_builtin().native_class.clone();
     while let Some(p) = native_rect.parent.clone() {
         native_rect = p;
     }
 
     crate::object_tree::recurse_elem_including_sub_components(
-        &root_component,
+        root_component,
         &(),
         &mut |elem: &ElementRc, _| {
-            if !matches!(elem.borrow().builtin_type(), Some(n) if n.name == "Flickable") {
+            if !matches!(&elem.borrow().base_type, Type::Builtin(n) if n.name == "Flickable") {
                 return;
             }
 
@@ -51,7 +51,7 @@ fn create_viewport_element(flickable_elem: &ElementRc, native_rect: &Rc<NativeCl
     let mut flickable = flickable_elem.borrow_mut();
     let flickable = &mut *flickable;
     let viewport = Rc::new(RefCell::new(Element {
-        id: format!("{}_viewport", flickable.id),
+        id: format!("{}-viewport", flickable.id),
         base_type: Type::Native(native_rect.clone()),
         children: std::mem::take(&mut flickable.children),
         enclosing_component: flickable.enclosing_component.clone(),
@@ -59,7 +59,7 @@ fn create_viewport_element(flickable_elem: &ElementRc, native_rect: &Rc<NativeCl
         ..Element::default()
     }));
     for (prop, info) in &flickable.base_type.as_builtin().properties {
-        if let Some(vp_prop) = prop.strip_prefix("viewport_") {
+        if let Some(vp_prop) = prop.strip_prefix("viewport-") {
             let nr = NamedReference::new(&viewport, vp_prop);
             flickable.property_declarations.insert(prop.to_owned(), info.ty.clone().into());
             match flickable.bindings.entry(prop.to_owned()) {
@@ -74,13 +74,9 @@ fn create_viewport_element(flickable_elem: &ElementRc, native_rect: &Rc<NativeCl
                     entry.insert(Expression::TwoWayBinding(nr, None).into());
                 }
             }
-            // Workaround for https://github.com/sixtyfpsui/sixtyfps/issues/193
-            if let Some(a) = flickable.property_animations.remove(prop) {
-                viewport.borrow_mut().property_animations.insert(vp_prop.into(), a);
-            }
         }
     }
-    flickable.children.push(viewport.clone());
+    flickable.children.push(viewport);
 }
 
 fn fixup_geometry(flickable_elem: &ElementRc) {
@@ -91,41 +87,47 @@ fn fixup_geometry(flickable_elem: &ElementRc) {
                 .children
                 .iter()
                 .filter(|x| is_layout(&x.borrow().base_type))
+                // FIXME: we should ideally add runtime code to merge layout info of all elements that are repeated (#407)
+                .filter(|x| x.borrow().repeated.is_none())
                 .map(|x| Expression::PropertyReference(NamedReference::new(x, prop)))
                 .fold1(|lhs, rhs| crate::builtin_macros::min_max_expression(lhs, rhs, op))
         })
     };
 
     if !flickable_elem.borrow().bindings.contains_key("height") {
-        forward_minmax_of("max_height", '<');
-        forward_minmax_of("preferred_height", '<');
+        forward_minmax_of("max-height", '<');
+        forward_minmax_of("preferred-height", '<');
     }
     if !flickable_elem.borrow().bindings.contains_key("width") {
-        forward_minmax_of("max_width", '<');
-        forward_minmax_of("preferred_width", '<');
+        forward_minmax_of("max-width", '<');
+        forward_minmax_of("preferred-width", '<');
     }
-    set_binding_if_not_explicit(flickable_elem, "viewport_width", || {
+    set_binding_if_not_explicit(flickable_elem, "viewport-width", || {
         Some(
             flickable_elem
                 .borrow()
                 .children
                 .iter()
                 .filter(|x| is_layout(&x.borrow().base_type))
-                .map(|x| Expression::PropertyReference(NamedReference::new(x, "min_width")))
+                // FIXME: (#407)
+                .filter(|x| x.borrow().repeated.is_none())
+                .map(|x| Expression::PropertyReference(NamedReference::new(x, "min-width")))
                 .fold(
                     Expression::PropertyReference(NamedReference::new(flickable_elem, "width")),
                     |lhs, rhs| crate::builtin_macros::min_max_expression(lhs, rhs, '>'),
                 ),
         )
     });
-    set_binding_if_not_explicit(flickable_elem, "viewport_height", || {
+    set_binding_if_not_explicit(flickable_elem, "viewport-height", || {
         Some(
             flickable_elem
                 .borrow()
                 .children
                 .iter()
                 .filter(|x| is_layout(&x.borrow().base_type))
-                .map(|x| Expression::PropertyReference(NamedReference::new(x, "min_height")))
+                // FIXME: (#407)
+                .filter(|x| x.borrow().repeated.is_none())
+                .map(|x| Expression::PropertyReference(NamedReference::new(x, "min-height")))
                 .fold(
                     Expression::PropertyReference(NamedReference::new(flickable_elem, "height")),
                     |lhs, rhs| crate::builtin_macros::min_max_expression(lhs, rhs, '>'),
