@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -60,9 +60,6 @@ fn inline_element(
     elem_mut.property_declarations.extend(
         inlined_component.root_element.borrow().property_declarations.iter().map(clone_tuple),
     );
-    elem_mut.property_animations.extend(
-        inlined_component.root_element.borrow().property_animations.iter().map(clone_tuple),
-    );
 
     for (p, a) in inlined_component.root_element.borrow().property_analysis.borrow().iter() {
         elem_mut.property_analysis.borrow_mut().entry(p.clone()).or_default().merge(a);
@@ -117,8 +114,9 @@ fn inline_element(
                 entry.insert(val.clone()).priority += 1;
             }
             std::collections::btree_map::Entry::Occupied(mut entry) => {
-                let b = entry.get_mut();
-                maybe_merge_two_ways(&mut b.expression, &mut b.priority, val);
+                if entry.get_mut().merge_with(val) {
+                    entry.get_mut().priority += 1;
+                }
             }
         }
     }
@@ -132,28 +130,6 @@ fn inline_element(
     }
 }
 
-/// Normally, binding would be kept intact. But if they are two ways binding, they need to be merged
-pub fn maybe_merge_two_ways(
-    binding: &mut Expression,
-    priority: &mut i32,
-    original: &BindingExpression,
-) {
-    match (binding, &original.expression) {
-        (Expression::TwoWayBinding(_, Some(ref mut x)), _) => {
-            maybe_merge_two_ways(&mut *x, priority, original);
-        }
-        (Expression::TwoWayBinding(_, x), o) => {
-            *priority = original.priority + 1;
-            *x = Some(Box::new(o.clone()));
-        }
-        (ref mut x, Expression::TwoWayBinding(nr, _)) => {
-            let n = Expression::TwoWayBinding(nr.clone(), Some(Box::new(std::mem::take(*x))));
-            **x = n
-        }
-        _ => *priority += 1,
-    }
-}
-
 // Duplicate the element elem and all its children. And fill the mapping to point from the old to the new
 fn duplicate_element_with_mapping(
     element: &ElementRc,
@@ -165,13 +141,12 @@ fn duplicate_element_with_mapping(
         base_type: elem.base_type.clone(),
         id: elem.id.clone(),
         property_declarations: elem.property_declarations.clone(),
-        property_animations: elem
-            .property_animations
-            .iter()
-            .map(|(k, v)| (k.clone(), duplicate_property_animation(v, mapping, root_component)))
-            .collect(),
         // We will do the fixup of the references in bindings later
-        bindings: elem.bindings.iter().map(increment_priority).collect(),
+        bindings: elem
+            .bindings
+            .iter()
+            .map(|b| duplicate_binding(b, mapping, root_component))
+            .collect(),
         property_analysis: elem.property_analysis.clone(),
         children: elem
             .children
@@ -198,9 +173,22 @@ fn duplicate_element_with_mapping(
 }
 
 /// Clone and increase the priority of a binding
-fn increment_priority((k, b): (&String, &BindingExpression)) -> (String, BindingExpression) {
-    let mut b = b.clone();
-    b.priority += 1;
+/// and duplicate its animation
+fn duplicate_binding(
+    (k, b): (&String, &BindingExpression),
+    mapping: &mut HashMap<ByAddress<ElementRc>, ElementRc>,
+    root_component: &Rc<Component>,
+) -> (String, BindingExpression) {
+    let b = BindingExpression {
+        expression: b.expression.clone(),
+        span: b.span.clone(),
+        priority: b.priority + 1,
+        animation: b
+            .animation
+            .as_ref()
+            .map(|pa| duplicate_property_animation(pa, mapping, root_component)),
+        analysis: b.analysis.clone(),
+    };
     (k.clone(), b)
 }
 
@@ -233,7 +221,7 @@ fn duplicate_property_animation(
 
 fn fixup_reference(nr: &mut NamedReference, mapping: &HashMap<ByAddress<ElementRc>, ElementRc>) {
     if let Some(e) = mapping.get(&element_key(nr.element())) {
-        *nr = NamedReference::new(&e, nr.name());
+        *nr = NamedReference::new(e, nr.name());
     }
 }
 
