@@ -1,13 +1,13 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
     Please contact info@sixtyfps.io for more information.
 LICENSE END */
-//! Passe that compute the layout constraint
+//! Passe that transform the PopupWindow element into a component
 
 use crate::diagnostics::BuildDiagnostics;
 use crate::expression_tree::{Expression, NamedReference};
@@ -16,7 +16,6 @@ use crate::object_tree::*;
 use crate::typeregister::TypeRegister;
 use std::rc::Rc;
 
-/// Currently this just removes the layout from the tree
 pub fn lower_popups(
     component: &Rc<Component>,
     type_register: &TypeRegister,
@@ -124,7 +123,7 @@ fn create_coordinate(
         };
     }
     let parent_element = parent_stack.last().unwrap();
-    let property_name = format!("{}_popup_{}", popup_comp.root_element.borrow().id, coord);
+    let property_name = format!("{}-popup-{}", popup_comp.root_element.borrow().id, coord);
     parent_element
         .borrow_mut()
         .property_declarations
