@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -43,23 +43,23 @@ fn create_box_shadow_element(
     }
 
     let mut element = Element {
-        id: format!("{}_shadow", sibling_element.borrow().id),
+        id: format!("{}-shadow", sibling_element.borrow().id),
         base_type: type_register.lookup_element("BoxShadow").unwrap(),
         enclosing_component: sibling_element.borrow().enclosing_component.clone(),
         bindings: shadow_property_bindings
             .into_iter()
             .map(|(shadow_prop_name, expr)| {
-                (shadow_prop_name.strip_prefix("drop_shadow_").unwrap().to_string(), expr)
+                (shadow_prop_name.strip_prefix("drop-shadow-").unwrap().to_string(), expr)
             })
             .collect(),
         ..Default::default()
     };
 
-    // FIXME: remove the border_radius manual mapping.
-    if sibling_element.borrow().bindings.contains_key("border_radius") {
+    // FIXME: remove the border-radius manual mapping.
+    if sibling_element.borrow().bindings.contains_key("border-radius") {
         element.bindings.insert(
-            "border_radius".to_string(),
-            Expression::PropertyReference(NamedReference::new(sibling_element, "border_radius"))
+            "border-radius".to_string(),
+            Expression::PropertyReference(NamedReference::new(sibling_element, "border-radius"))
                 .into(),
         );
     }
@@ -136,13 +136,13 @@ pub fn lower_shadow_properties(
     for (shadow_prop_name, shadow_prop_binding) in
         take_shadow_property_bindings(&component.root_element)
     {
-        diag.push_error(
-            format!("The {} property cannot be used on the root element", shadow_prop_name),
+        diag.push_warning(
+            format!("The {} property cannot be used on the root element, the shadow will not be visible", shadow_prop_name),
             &shadow_prop_binding,
         );
     }
 
-    recurse_elem_including_sub_components_no_borrow(&component, &(), &mut |elem, _| {
+    recurse_elem_including_sub_components_no_borrow(component, &(), &mut |elem, _| {
         // When encountering a repeater where the repeated element has a `drop-shadow` property, we create a new
         // dedicated shadow element and make the previously repeated element a child of that. This ensures rendering
         // underneath while maintaining the hierarchy for the repeater.
@@ -189,11 +189,11 @@ pub fn lower_shadow_properties(
                 // original, such as x/y/width/height.
                 for (prop, _) in crate::typeregister::RESERVED_GEOMETRY_PROPERTIES.iter() {
                     let prop = prop.to_string();
-                    if !shadow_elem.bindings.contains_key(&prop) {
+                    shadow_elem.bindings.entry(prop.clone()).or_insert_with(|| {
                         let binding_ref =
                             Expression::PropertyReference(NamedReference::new(&child, &prop));
-                        shadow_elem.bindings.insert(prop, binding_ref.into());
-                    }
+                        binding_ref.into()
+                    });
                 }
 
                 elem.borrow_mut().children.push(ElementRc::new(RefCell::new(shadow_elem)));
