@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -44,7 +44,7 @@ fn lower_state_in_element(
     let state_property_ref = if has_transitions {
         Expression::StructFieldAccess {
             base: Box::new(state_property.clone()),
-            name: "current_state".into(),
+            name: "current-state".into(),
         }
     } else {
         state_property.clone()
@@ -62,10 +62,17 @@ fn lower_state_in_element(
                 false_expr: Box::new(std::mem::take(&mut state_value)),
             };
         }
-        for (ne, expr) in state.property_changes {
+        for (ne, expr, node) in state.property_changes {
             affected_properties.insert(ne.clone());
             let e = ne.element();
             let property_expr = expression_for_property(&e, ne.name());
+            if matches!(property_expr, Expression::TwoWayBinding(..)) {
+                diag.push_error(
+                    format!("Cannot change the property '{}' in a state because it is initialized with a two-way binding", ne.name()),
+                    &node
+                );
+                continue;
+            }
             let new_expr = Expression::Condition {
                 condition: Box::new(Expression::BinaryExpression {
                     lhs: Box::new(state_property_ref.clone()),
@@ -130,6 +137,7 @@ fn lower_transitions_in_element(
                     "The property is not changed as part of this transition".into(),
                     &span,
                 );
+                continue;
             }
 
             let t = TransitionPropertyAnimation {
@@ -142,23 +150,19 @@ fn lower_transitions_in_element(
     }
     for (ne, (span, animations)) in props {
         let e = ne.element();
-        match e.borrow_mut().property_animations.entry(ne.name().to_owned()) {
-            std::collections::hash_map::Entry::Occupied(e) => {
-                diag.push_error(
-                    format!(
+        // We check earlier that the property is in the set of changed properties, so a binding bust have been assigned
+        let old_anim = e.borrow_mut().bindings.get_mut(ne.name()).unwrap().animation.replace(
+            PropertyAnimation::Transition { state_ref: state_property.clone(), animations },
+        );
+        if old_anim.is_some() {
+            diag.push_error(
+                format!(
                     "The property '{}' cannot have transition because it already has an animation",
-                    e.key()
+                    ne.name()
                 ),
-                    &span,
-                );
-            }
-            std::collections::hash_map::Entry::Vacant(e) => {
-                e.insert(PropertyAnimation::Transition {
-                    state_ref: state_property.clone(),
-                    animations,
-                });
-            }
-        };
+                &span,
+            );
+        }
     }
 }
 
@@ -168,7 +172,7 @@ fn compute_state_property_name(root_element: &ElementRc) -> String {
     while root_element.borrow().lookup_property(property_name.as_ref()).property_type
         != Type::Invalid
     {
-        property_name += "_";
+        property_name += "-";
     }
     property_name
 }
@@ -178,7 +182,9 @@ fn expression_for_property(element: &ElementRc, name: &str) -> Expression {
     let mut element_it = Some(element.clone());
     while let Some(element) = element_it {
         if let Some(e) = element.borrow().bindings.get(name) {
-            return e.expression.clone();
+            if !matches!(e.expression, Expression::Invalid) {
+                return e.expression.clone();
+            }
         }
         element_it = if let Type::Component(base) = &element.borrow().base_type {
             Some(base.root_element.clone())
