@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -34,7 +34,6 @@ pub fn move_declarations(component: &Rc<Component>, diag: &mut BuildDiagnostics)
     component.popup_windows.borrow().iter().for_each(|f| move_declarations(&f.component, diag));
 
     let mut new_root_bindings = HashMap::new();
-    let mut new_root_property_animations = HashMap::new();
     let mut new_root_property_analysis = HashMap::new();
 
     let move_bindings_and_animations = &mut |elem: &ElementRc| {
@@ -55,7 +54,7 @@ pub fn move_declarations(component: &Rc<Component>, diag: &mut BuildDiagnostics)
 
         // take the bindings so we do nt keep the borrow_mut of the element
         let bindings = core::mem::take(&mut elem.borrow_mut().bindings);
-        let mut new_bindings = BTreeMap::new();
+        let mut new_bindings = BindingsMap::default();
         for (k, e) in bindings {
             let will_be_moved = elem.borrow().property_declarations.contains_key(&k);
             if will_be_moved {
@@ -66,18 +65,6 @@ pub fn move_declarations(component: &Rc<Component>, diag: &mut BuildDiagnostics)
         }
         elem.borrow_mut().bindings = new_bindings;
 
-        let property_animations = core::mem::take(&mut elem.borrow_mut().property_animations);
-        let mut new_property_animations = HashMap::with_capacity(property_animations.len());
-        for (anim_prop, anim) in property_animations {
-            let will_be_moved = elem.borrow().property_declarations.contains_key(&anim_prop);
-            if will_be_moved {
-                new_root_property_animations.insert(map_name(elem, anim_prop.as_str()), anim);
-            } else {
-                new_property_animations.insert(anim_prop, anim);
-            }
-        }
-        elem.borrow_mut().property_animations = new_property_animations;
-
         let property_analysis = elem.borrow().property_analysis.take();
         let mut new_property_analysis = HashMap::with_capacity(property_analysis.len());
         for (prop, a) in property_analysis {
@@ -117,7 +104,6 @@ pub fn move_declarations(component: &Rc<Component>, diag: &mut BuildDiagnostics)
         let mut r = component.root_element.borrow_mut();
         r.property_declarations = decl.property_declarations;
         r.bindings.extend(new_root_bindings.into_iter());
-        r.property_animations.extend(new_root_property_animations.into_iter());
         r.property_analysis.borrow_mut().extend(new_root_property_analysis.into_iter());
     }
 
@@ -138,7 +124,7 @@ fn fixup_reference(nr: &mut NamedReference) {
 }
 
 fn map_name(e: &ElementRc, s: &str) -> String {
-    format!("{}_{}", e.borrow().id, s)
+    format!("{}-{}", e.borrow().id, s)
 }
 
 /// Optimized item are not used for the fact that they are items, but their properties
