@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -77,7 +77,7 @@ pub fn remove_aliases(component: &Rc<Component>, diag: &mut BuildDiagnostics) {
                     diag.push_error("Property cannot alias to itself".into(), binding);
                     continue 'bindings;
                 }
-                property_sets.add_link(NamedReference::new(e, &name), nr.clone());
+                property_sets.add_link(NamedReference::new(e, name), nr.clone());
 
                 exp = match next {
                     Some(x) => &*x,
@@ -108,7 +108,7 @@ pub fn remove_aliases(component: &Rc<Component>, diag: &mut BuildDiagnostics) {
     }
 
     // Do the replacements
-    visit_all_named_references(&component, &mut |nr: &mut NamedReference| {
+    visit_all_named_references(component, &mut |nr: &mut NamedReference| {
         if let Some(new) = aliases_to_remove.get(nr) {
             *nr = new.clone();
         }
@@ -127,20 +127,12 @@ pub fn remove_aliases(component: &Rc<Component>, diag: &mut BuildDiagnostics) {
                 match to_elem.borrow_mut().bindings.entry(to.name().to_owned()) {
                     Entry::Occupied(mut e) => {
                         simplify_expression(e.get_mut(), &to);
-                        if matches!(e.get().expression, Expression::Invalid) {
-                            *e.get_mut() = binding;
-                        } else if e.get().priority < binding.priority {
-                            crate::passes::inlining::maybe_merge_two_ways(
-                                &mut e.get_mut().expression,
-                                &mut 0,
-                                &binding,
-                            );
+                        if e.get().priority < binding.priority
+                            || matches!(e.get().expression, Expression::Invalid)
+                        {
+                            e.get_mut().merge_with(&binding);
                         } else {
-                            crate::passes::inlining::maybe_merge_two_ways(
-                                &mut binding.expression,
-                                &mut 0,
-                                &e.get(),
-                            );
+                            binding.merge_with(e.get());
                             *e.get_mut() = binding;
                         }
                     }
@@ -173,9 +165,8 @@ pub fn remove_aliases(component: &Rc<Component>, diag: &mut BuildDiagnostics) {
             if let Some(d) = elem.property_declarations.get_mut(remove.name()) {
                 if d.expose_in_public_api {
                     d.is_alias = Some(to.clone());
-                    drop(d);
                     drop(elem);
-                    // one must mark the aliased property as setable from outside
+                    // one must mark the aliased property as settable from outside
                     to.element()
                         .borrow()
                         .property_analysis
@@ -184,7 +175,6 @@ pub fn remove_aliases(component: &Rc<Component>, diag: &mut BuildDiagnostics) {
                         .or_default()
                         .is_set = true;
                 } else {
-                    drop(d);
                     elem.property_declarations.remove(remove.name());
                 }
             } else {
@@ -201,7 +191,7 @@ fn is_declaration(x: &NamedReference) -> bool {
 }
 
 /// Out of two named reference, return the one which is the best to keep.
-fn best_property<'a>(
+fn best_property(
     component: &Rc<Component>,
     p1: NamedReference,
     p2: NamedReference,
