@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -11,7 +11,7 @@ LICENSE END */
 Make sure that the Repeated expression are just components without any children
  */
 
-use crate::expression_tree::NamedReference;
+use crate::expression_tree::{Expression, NamedReference};
 use crate::langtype::Type;
 use crate::object_tree::*;
 use std::cell::RefCell;
@@ -24,9 +24,10 @@ pub fn process_repeater_components(component: &Rc<Component>) {
 
 fn create_repeater_components(component: &Rc<Component>) {
     recurse_elem(&component.root_element, &(), &mut |elem, _| {
-        if elem.borrow().repeated.is_none() {
-            return;
-        }
+        let is_listview = match &elem.borrow().repeated {
+            Some(r) => r.is_listview.clone(),
+            None => return,
+        };
         let parent_element = Rc::downgrade(elem);
         let mut elem = elem.borrow_mut();
 
@@ -38,14 +39,13 @@ fn create_repeater_components(component: &Rc<Component>) {
                 property_analysis: std::mem::take(&mut elem.property_analysis),
                 children: std::mem::take(&mut elem.children),
                 property_declarations: std::mem::take(&mut elem.property_declarations),
-                property_animations: std::mem::take(&mut elem.property_animations),
                 named_references: Default::default(),
                 repeated: None,
                 node: elem.node.clone(),
                 enclosing_component: Default::default(),
                 states: std::mem::take(&mut elem.states),
                 transitions: std::mem::take(&mut elem.transitions),
-                child_of_layout: elem.child_of_layout,
+                child_of_layout: elem.child_of_layout || is_listview.is_some(),
                 layout_info_prop: elem.layout_info_prop.take(),
                 is_flickable_viewport: elem.is_flickable_viewport,
                 item_index: Default::default(), // Not determined yet
@@ -54,6 +54,22 @@ fn create_repeater_components(component: &Rc<Component>) {
             ..Component::default()
         });
 
+        if let Some(listview) = is_listview {
+            if !comp.root_element.borrow().bindings.contains_key("height") {
+                let preferred = Expression::PropertyReference(NamedReference::new(
+                    &comp.root_element,
+                    "preferred-height",
+                ));
+                comp.root_element.borrow_mut().bindings.insert("height".into(), preferred.into());
+            }
+            if !comp.root_element.borrow().bindings.contains_key("width") {
+                comp.root_element.borrow_mut().bindings.insert(
+                    "width".into(),
+                    Expression::PropertyReference(listview.listview_width).into(),
+                );
+            }
+        }
+
         let weak = Rc::downgrade(&comp);
         recurse_elem(&comp.root_element, &(), &mut |e, _| {
             e.borrow_mut().enclosing_component = weak.clone()
@@ -66,7 +82,7 @@ fn create_repeater_components(component: &Rc<Component>) {
 /// Make sure that references to property within the repeated element actually point to the reference
 /// to the root of the newly created component
 fn adjust_references(comp: &Rc<Component>) {
-    visit_all_named_references(&comp, &mut |nr| {
+    visit_all_named_references(comp, &mut |nr| {
         if nr.name() == "$model" {
             return;
         }
