@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -99,11 +99,15 @@ pub fn resolve_expressions(
                 if is_repeated {
                     // The first expression is always the model and it needs to be resolved with the parent scope
                     debug_assert!(elem.borrow().repeated.as_ref().is_none()); // should be none because it is taken by the visit_element_expressions function
+                    let mut parent_scope = scope.clone();
+                    if let Some(parent) = find_parent_element(elem) {
+                        parent_scope.0.push(parent)
+                    };
                     resolve_expression(
                         expr,
                         property_name,
                         property_type(),
-                        scope,
+                        &parent_scope,
                         &doc.local_registry,
                         type_loader,
                         diag,
@@ -203,7 +207,7 @@ impl Expression {
     ) -> Expression {
         let return_type = ctx.return_type().clone();
         Expression::ReturnStatement(node.Expression().map(|n| {
-            Box::new(Self::from_expression_node(n.into(), ctx).maybe_convert_to(
+            Box::new(Self::from_expression_node(n, ctx).maybe_convert_to(
                 return_type,
                 &node,
                 &mut ctx.diag,
@@ -343,7 +347,7 @@ impl Expression {
                             .to_string_lossy()
                             .to_string()
                     })
-                    .unwrap_or_else(|| s.into())
+                    .unwrap_or(s)
             }
         };
 
@@ -392,7 +396,7 @@ impl Expression {
                     Stop::Finished => {}
                     Stop::Color(col) => stops.push((
                         col,
-                        if stops.len() == 0 {
+                        if stops.is_empty() {
                             Expression::NumberLiteral(0., Unit::None)
                         } else {
                             Expression::Invalid
@@ -423,8 +427,14 @@ impl Expression {
                 }
             }
         }
-        if let Stop::Color(col) = current_stop {
-            stops.push((col, Expression::NumberLiteral(1., Unit::None)))
+        match current_stop {
+            Stop::Color(col) => stops.push((col, Expression::NumberLiteral(1., Unit::None))),
+            Stop::Empty => {
+                if let Some((_, e @ Expression::Invalid)) = stops.last_mut() {
+                    *e = Expression::NumberLiteral(1., Unit::None)
+                }
+            }
+            Stop::Finished => (),
         };
 
         // Fix the stop so each has a position.
@@ -492,9 +502,12 @@ impl Expression {
         let result = match global_lookup.lookup(ctx, &first_str) {
             None => {
                 if let Some(minus_pos) = first.text().find('-') {
-                    // Attempt to recover if the user wanted to write "-"
+                    // Attempt to recover if the user wanted to write "-" for minus
                     let first_str = &first.text()[0..minus_pos];
-                    if global_lookup.lookup(ctx, first_str).is_some() {
+                    if global_lookup
+                        .lookup(ctx, &crate::parser::normalize_identifier(first_str))
+                        .is_some()
+                    {
                         ctx.diag.push_error(format!("Unknown unqualified identifier '{}'. Use space before the '-' if you meant a subtraction", first.text()), &node);
                         return Expression::Invalid;
                     }
@@ -519,13 +532,13 @@ impl Expression {
 
         match result.expression {
             Expression::ElementReference(e) => {
-                return continue_lookup_within_element(&e.upgrade().unwrap(), &mut it, node, ctx);
+                continue_lookup_within_element(&e.upgrade().unwrap(), &mut it, node, ctx)
             }
             r @ Expression::CallbackReference(..) => {
                 if let Some(x) = it.next() {
                     ctx.diag.push_error("Cannot access fields of callback".into(), &x)
                 }
-                return r;
+                r
             }
             Expression::EnumerationValue(ev) => {
                 // Special meaning for the type itself
@@ -549,13 +562,13 @@ impl Expression {
                             Some(r) => return maybe_lookup_object(r.expression, it, ctx),
                         }
                     } else {
-                        ctx.diag.push_error(format!("Cannot take reference to an enum",), &node);
+                        ctx.diag.push_error("Cannot take reference to an enum".to_string(), &node);
                         return Expression::Invalid;
                     }
                 }
-                return maybe_lookup_object(Expression::EnumerationValue(ev), it, ctx);
+                maybe_lookup_object(Expression::EnumerationValue(ev), it, ctx)
             }
-            other => return maybe_lookup_object(other, it, ctx),
+            other => maybe_lookup_object(other, it, ctx),
         }
     }
 
@@ -628,13 +641,13 @@ impl Expression {
         ctx: &mut LookupCtx,
     ) -> Expression {
         let (lhs_n, rhs_n) = node.Expression();
-        let mut lhs = Self::from_expression_node(lhs_n, ctx);
+        let mut lhs = Self::from_expression_node(lhs_n.clone(), ctx);
         let op = None
-            .or(node.child_token(SyntaxKind::PlusEqual).and(Some('+')))
-            .or(node.child_token(SyntaxKind::MinusEqual).and(Some('-')))
-            .or(node.child_token(SyntaxKind::StarEqual).and(Some('*')))
-            .or(node.child_token(SyntaxKind::DivEqual).and(Some('/')))
-            .or(node.child_token(SyntaxKind::Equal).and(Some('=')))
+            .or_else(|| node.child_token(SyntaxKind::PlusEqual).and(Some('+')))
+            .or_else(|| node.child_token(SyntaxKind::MinusEqual).and(Some('-')))
+            .or_else(|| node.child_token(SyntaxKind::StarEqual).and(Some('*')))
+            .or_else(|| node.child_token(SyntaxKind::DivEqual).and(Some('/')))
+            .or_else(|| node.child_token(SyntaxKind::Equal).and(Some('=')))
             .unwrap_or('_');
         if !lhs.try_set_rw() && lhs.ty() != Type::Invalid {
             ctx.diag.push_error(
@@ -645,12 +658,28 @@ impl Expression {
                 &node,
             );
         }
-        let rhs = Self::from_expression_node(rhs_n.clone(), ctx).maybe_convert_to(
-            lhs.ty(),
-            &rhs_n,
-            &mut ctx.diag,
-        );
-        Expression::SelfAssignment { lhs: Box::new(lhs), rhs: Box::new(rhs), op }
+        let ty = lhs.ty();
+        let expected_ty = match op {
+            '=' => ty,
+            '+' if ty == Type::String || ty.as_unit_product().is_some() => ty,
+            '-' if ty.as_unit_product().is_some() => ty,
+            '/' | '*' if ty.as_unit_product().is_some() => Type::Float32,
+            _ => {
+                if ty != Type::Invalid {
+                    ctx.diag.push_error(
+                        format!("the {}= operation cannot be done on a {}", op, ty),
+                        &lhs_n,
+                    );
+                }
+                Type::Invalid
+            }
+        };
+        let rhs = Self::from_expression_node(rhs_n.clone(), ctx);
+        Expression::SelfAssignment {
+            lhs: Box::new(lhs),
+            rhs: Box::new(rhs.maybe_convert_to(expected_ty, &rhs_n, &mut ctx.diag)),
+            op,
+        }
     }
 
     fn from_binary_expression_node(
@@ -658,18 +687,18 @@ impl Expression {
         ctx: &mut LookupCtx,
     ) -> Expression {
         let op = None
-            .or(node.child_token(SyntaxKind::Plus).and(Some('+')))
-            .or(node.child_token(SyntaxKind::Minus).and(Some('-')))
-            .or(node.child_token(SyntaxKind::Star).and(Some('*')))
-            .or(node.child_token(SyntaxKind::Div).and(Some('/')))
-            .or(node.child_token(SyntaxKind::LessEqual).and(Some('≤')))
-            .or(node.child_token(SyntaxKind::GreaterEqual).and(Some('≥')))
-            .or(node.child_token(SyntaxKind::LAngle).and(Some('<')))
-            .or(node.child_token(SyntaxKind::RAngle).and(Some('>')))
-            .or(node.child_token(SyntaxKind::EqualEqual).and(Some('=')))
-            .or(node.child_token(SyntaxKind::NotEqual).and(Some('!')))
-            .or(node.child_token(SyntaxKind::AndAnd).and(Some('&')))
-            .or(node.child_token(SyntaxKind::OrOr).and(Some('|')))
+            .or_else(|| node.child_token(SyntaxKind::Plus).and(Some('+')))
+            .or_else(|| node.child_token(SyntaxKind::Minus).and(Some('-')))
+            .or_else(|| node.child_token(SyntaxKind::Star).and(Some('*')))
+            .or_else(|| node.child_token(SyntaxKind::Div).and(Some('/')))
+            .or_else(|| node.child_token(SyntaxKind::LessEqual).and(Some('≤')))
+            .or_else(|| node.child_token(SyntaxKind::GreaterEqual).and(Some('≥')))
+            .or_else(|| node.child_token(SyntaxKind::LAngle).and(Some('<')))
+            .or_else(|| node.child_token(SyntaxKind::RAngle).and(Some('>')))
+            .or_else(|| node.child_token(SyntaxKind::EqualEqual).and(Some('=')))
+            .or_else(|| node.child_token(SyntaxKind::NotEqual).and(Some('!')))
+            .or_else(|| node.child_token(SyntaxKind::AndAnd).and(Some('&')))
+            .or_else(|| node.child_token(SyntaxKind::OrOr).and(Some('|')))
             .unwrap_or('_');
 
         let (lhs_n, rhs_n) = node.Expression();
@@ -755,9 +784,9 @@ impl Expression {
         Expression::UnaryOp {
             sub: Box::new(exp),
             op: None
-                .or(node.child_token(SyntaxKind::Plus).and(Some('+')))
-                .or(node.child_token(SyntaxKind::Minus).and(Some('-')))
-                .or(node.child_token(SyntaxKind::Bang).and(Some('!')))
+                .or_else(|| node.child_token(SyntaxKind::Plus).and(Some('+')))
+                .or_else(|| node.child_token(SyntaxKind::Minus).and(Some('-')))
+                .or_else(|| node.child_token(SyntaxKind::Bang).and(Some('!')))
                 .unwrap_or('_'),
         }
     }
@@ -839,7 +868,7 @@ impl Expression {
             )
         });
         let mut result = exprs.next().unwrap_or_default();
-        while let Some(x) = exprs.next() {
+        for x in exprs {
             result = Expression::BinaryExpression {
                 lhs: Box::new(std::mem::take(&mut result)),
                 rhs: Box::new(x),
@@ -892,12 +921,11 @@ impl Expression {
                     (target_type, expr_ty) => {
                         if expr_ty.can_convert(&target_type) {
                             target_type
-                        } else if target_type.can_convert(&expr_ty) {
-                            expr_ty
-                        } else if expr_ty.default_unit().is_some()
-                            && matches!(target_type, Type::Float32 | Type::Int32)
+                        } else if target_type.can_convert(&expr_ty)
+                            || (expr_ty.default_unit().is_some()
+                                && matches!(target_type, Type::Float32 | Type::Int32))
                         {
-                            // in case this is the '0' literal
+                            // in the or case: The `0` literal.
                             expr_ty
                         } else {
                             // otherwise, use the target type and let further conversion report an error
@@ -962,16 +990,19 @@ fn continue_lookup_within_element(
                 }
             };
             ctx.diag.push_error(
-                format!("{} does not have a property '{}'.{}", what, second.text(), extra),
+                format!("{} does not have a property '{}'{}", what, second.text(), extra),
                 &second,
             );
         };
         if let Some(minus_pos) = second.text().find('-') {
             // Attempt to recover if the user wanted to write "-"
-            if elem.borrow().lookup_property(&second.text()[0..minus_pos]).property_type
+            if elem
+                .borrow()
+                .lookup_property(&crate::parser::normalize_identifier(&second.text()[0..minus_pos]))
+                .property_type
                 != Type::Invalid
             {
-                err(" Use space before the '-' if you meant a subtraction");
+                err(". Use space before the '-' if you meant a subtraction");
                 return Expression::Invalid;
             }
         }
