@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -15,6 +15,7 @@ use std::rc::Rc;
 
 use crate::diagnostics::{BuildDiagnostics, Spanned};
 use crate::object_tree::{self, Document};
+use crate::parser;
 use crate::parser::{syntax_nodes, NodeOrToken, SyntaxKind, SyntaxToken};
 use crate::typeregister::TypeRegister;
 use crate::CompilerConfiguration;
@@ -58,10 +59,11 @@ impl ImportedName {
     }
 
     pub fn from_node(importident: syntax_nodes::ImportIdentifier) -> Self {
-        let external_name = importident.ExternalName().text().to_string().trim().to_string();
+        let external_name =
+            parser::normalize_identifier(importident.ExternalName().text().to_string().trim());
 
         let internal_name = match importident.InternalName() {
-            Some(name_ident) => name_ident.text().to_string().trim().to_string(),
+            Some(name_ident) => parser::normalize_identifier(name_ident.text().to_string().trim()),
             None => external_name.clone(),
         };
 
@@ -91,12 +93,12 @@ impl<'a> TypeLoader<'a> {
             let is_wasm = cfg!(target_arch = "wasm32")
                 || std::env::var("TARGET").map_or(false, |t| t.starts_with("wasm"));
             if !is_wasm {
-                diag.push_diagnostic_with_span("SIXTYFPS_STYLE not defined, defaulting to 'ugly', see https://github.com/sixtyfpsui/sixtyfps/issues/83 for more info".to_owned(),
+                diag.push_diagnostic_with_span("SIXTYFPS_STYLE not defined, defaulting to 'fluent', see https://github.com/sixtyfpsui/sixtyfps/issues/83 for more info".to_owned(),
                     Default::default(),
                     crate::diagnostics::DiagnosticLevel::Warning
                 );
             }
-            Cow::from("ugly")
+            Cow::from("fluent")
         });
 
         let builtin_library =
@@ -117,7 +119,7 @@ impl<'a> TypeLoader<'a> {
         diagnostics: &mut BuildDiagnostics,
         registry_to_populate: &Rc<RefCell<TypeRegister>>,
     ) -> Vec<ImportedTypes> {
-        let dependencies = self.collect_dependencies(&doc, diagnostics).await;
+        let dependencies = self.collect_dependencies(doc, diagnostics).await;
         let mut foreign_imports = vec![];
         for mut import in dependencies.into_iter() {
             if import.file.ends_with(".60") {
@@ -232,7 +234,8 @@ impl<'a> TypeLoader<'a> {
         }
 
         // Drop &self lifetime attached to is_builtin, in order to mutable borrow self below
-        let is_builtin = is_builtin.map(|s| s.to_owned());
+        let builtin = is_builtin.map(|s| s.to_owned());
+        let is_builtin = builtin.is_some();
 
         if !self.all_documents.currently_loading.insert(path_canon.clone()) {
             diagnostics
@@ -240,8 +243,8 @@ impl<'a> TypeLoader<'a> {
             return None;
         }
 
-        let source_code_result = if let Some(builtin) = is_builtin {
-            Ok(builtin.to_owned())
+        let source_code_result = if let Some(builtin) = builtin {
+            Ok(builtin)
         } else if let Some(fallback) = &self.compiler_config.open_import_fallback {
             let result = fallback(path_canon.to_string_lossy().into()).await;
             result.unwrap_or_else(|| std::fs::read_to_string(&path_canon))
@@ -270,7 +273,7 @@ impl<'a> TypeLoader<'a> {
             }
         };
 
-        self.load_file(&path_canon, &path, source_code, diagnostics).await;
+        self.load_file(&path_canon, &path, source_code, is_builtin, diagnostics).await;
         let _ok = self.all_documents.currently_loading.remove(path_canon.as_path());
         assert!(_ok);
         Some(path_canon)
@@ -284,13 +287,15 @@ impl<'a> TypeLoader<'a> {
         path: &Path,
         source_path: &Path,
         source_code: String,
+        is_builtin: bool,
         diagnostics: &mut BuildDiagnostics,
     ) {
         let dependency_doc: syntax_nodes::Document =
-            crate::parser::parse(source_code, Some(&source_path), diagnostics).into();
+            crate::parser::parse(source_code, Some(source_path), diagnostics).into();
 
         let dependency_registry =
             Rc::new(RefCell::new(TypeRegister::new(&self.global_type_registry)));
+        dependency_registry.borrow_mut().expose_internal_types = is_builtin;
         let foreign_imports = self
             .load_dependencies_recursively(&dependency_doc, diagnostics, &dependency_registry)
             .await;
@@ -319,9 +324,8 @@ impl<'a> TypeLoader<'a> {
             diagnostics,
             &dependency_registry,
         );
-        crate::passes::infer_aliases_types::resolve_aliases(&doc, diagnostics);
-        crate::passes::resolving::resolve_expressions(&doc, &self, diagnostics);
-        crate::passes::check_expressions::check_expressions(&doc, diagnostics);
+        crate::passes::run_import_passes(&doc, self, diagnostics);
+
         self.all_documents.docs.insert(path.to_owned(), doc);
     }
 
@@ -469,13 +473,13 @@ fn test_dependency_loading() {
     let mut compiler_config =
         CompilerConfiguration::new(crate::generator::OutputFormat::Interpreter);
     compiler_config.include_paths = vec![incdir];
-    compiler_config.style = Some("ugly".into());
+    compiler_config.style = Some("fluent".into());
 
-    let mut main_test_path = test_source_path.clone();
+    let mut main_test_path = test_source_path;
     main_test_path.push("dependency_test_main.60");
 
     let mut test_diags = crate::diagnostics::BuildDiagnostics::default();
-    let doc_node = crate::parser::parse_file(main_test_path.clone(), &mut test_diags).unwrap();
+    let doc_node = crate::parser::parse_file(main_test_path, &mut test_diags).unwrap();
 
     let doc_node: syntax_nodes::Document = doc_node.into();
 
@@ -504,12 +508,12 @@ fn test_load_from_callback_ok() {
 
     let mut compiler_config =
         CompilerConfiguration::new(crate::generator::OutputFormat::Interpreter);
-    compiler_config.style = Some("ugly".into());
+    compiler_config.style = Some("fluent".into());
     compiler_config.open_import_fallback = Some(Rc::new(move |path| {
         let ok_ = ok_.clone();
         Box::pin(async move {
             assert_eq!(path, "../FooBar.60");
-            assert_eq!(ok_.get(), false);
+            assert!(!ok_.get());
             ok_.set(true);
             Some(Ok("export XX := Rectangle {} ".to_owned()))
         })
@@ -523,7 +527,7 @@ import { XX } from "../FooBar.60";
 X := XX {}
 "#
         .into(),
-        Some(&std::path::Path::new("HELLO")),
+        Some(std::path::Path::new("HELLO")),
         &mut test_diags,
     );
 
@@ -537,7 +541,7 @@ X := XX {}
         &mut build_diagnostics,
         &registry,
     ));
-    assert_eq!(ok.get(), true);
+    assert!(ok.get());
     assert!(!test_diags.has_error());
     assert!(!build_diagnostics.has_error());
 }
@@ -546,7 +550,7 @@ X := XX {}
 fn test_manual_import() {
     let mut compiler_config =
         CompilerConfiguration::new(crate::generator::OutputFormat::Interpreter);
-    compiler_config.style = Some("ugly".into());
+    compiler_config.style = Some("fluent".into());
     let global_registry = TypeRegister::builtin();
     let mut build_diagnostics = BuildDiagnostics::default();
     let mut loader = TypeLoader::new(global_registry, &compiler_config, &mut build_diagnostics);
