@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -15,7 +15,7 @@ use crate::expression_tree::{BuiltinFunction, Expression};
 use crate::langtype::{BuiltinPropertyInfo, Enumeration, PropertyLookupResult, Type};
 use crate::object_tree::Component;
 
-pub(crate) const RESERVED_GEOMETRY_PROPERTIES: &'static [(&'static str, Type)] = &[
+pub(crate) const RESERVED_GEOMETRY_PROPERTIES: &[(&str, Type)] = &[
     ("x", Type::LogicalLength),
     ("y", Type::LogicalLength),
     ("width", Type::LogicalLength),
@@ -23,37 +23,37 @@ pub(crate) const RESERVED_GEOMETRY_PROPERTIES: &'static [(&'static str, Type)] =
     ("z", Type::Float32),
 ];
 
-const RESERVED_LAYOUT_PROPERTIES: &'static [(&'static str, Type)] = &[
-    ("min_width", Type::LogicalLength),
-    ("min_height", Type::LogicalLength),
-    ("max_width", Type::LogicalLength),
-    ("max_height", Type::LogicalLength),
+const RESERVED_LAYOUT_PROPERTIES: &[(&str, Type)] = &[
+    ("min-width", Type::LogicalLength),
+    ("min-height", Type::LogicalLength),
+    ("max-width", Type::LogicalLength),
+    ("max-height", Type::LogicalLength),
     ("padding", Type::LogicalLength),
-    ("padding_left", Type::LogicalLength),
-    ("padding_right", Type::LogicalLength),
-    ("padding_top", Type::LogicalLength),
-    ("padding_bottom", Type::LogicalLength),
-    ("preferred_width", Type::LogicalLength),
-    ("preferred_height", Type::LogicalLength),
-    ("horizontal_stretch", Type::Float32),
-    ("vertical_stretch", Type::Float32),
+    ("padding-left", Type::LogicalLength),
+    ("padding-right", Type::LogicalLength),
+    ("padding-top", Type::LogicalLength),
+    ("padding-bottom", Type::LogicalLength),
+    ("preferred-width", Type::LogicalLength),
+    ("preferred-height", Type::LogicalLength),
+    ("horizontal-stretch", Type::Float32),
+    ("vertical-stretch", Type::Float32),
     ("col", Type::Int32),
     ("row", Type::Int32),
     ("colspan", Type::Int32),
     ("rowspan", Type::Int32),
 ];
 
-const RESERVED_OTHER_PROPERTIES: &'static [(&'static str, Type)] = &[
+const RESERVED_OTHER_PROPERTIES: &[(&str, Type)] = &[
     ("clip", Type::Bool),
     ("opacity", Type::Float32),
     ("visible", Type::Bool), // ("enabled", Type::Bool),
 ];
 
-pub(crate) const RESERVED_DROP_SHADOW_PROPERTIES: &'static [(&'static str, Type)] = &[
-    ("drop_shadow_offset_x", Type::LogicalLength),
-    ("drop_shadow_offset_y", Type::LogicalLength),
-    ("drop_shadow_blur", Type::LogicalLength),
-    ("drop_shadow_color", Type::Color),
+pub(crate) const RESERVED_DROP_SHADOW_PROPERTIES: &[(&str, Type)] = &[
+    ("drop-shadow-offset-x", Type::LogicalLength),
+    ("drop-shadow-offset-y", Type::LogicalLength),
+    ("drop-shadow-blur", Type::LogicalLength),
+    ("drop-shadow-color", Type::Color),
 ];
 
 /// list of reserved property injected in every item
@@ -65,7 +65,7 @@ pub fn reserved_properties() -> impl Iterator<Item = (&'static str, Type)> {
         .chain(RESERVED_DROP_SHADOW_PROPERTIES.iter())
         .map(|(k, v)| (*k, v.clone()))
         .chain(std::array::IntoIter::new([
-            ("forward_focus", Type::ElementReference),
+            ("forward-focus", Type::ElementReference),
             ("focus", BuiltinFunction::SetFocusItem.ty()),
         ]))
 }
@@ -83,10 +83,10 @@ pub fn reserved_property(name: &str) -> PropertyLookupResult {
         if let Some(a) = name.strip_prefix(pre) {
             for suf in &["width", "height"] {
                 if let Some(b) = a.strip_suffix(suf) {
-                    if b == "imum_" {
+                    if b == "imum-" {
                         return PropertyLookupResult {
                             property_type: Type::LogicalLength,
-                            resolved_name: format!("{}_{}", pre, suf).into(),
+                            resolved_name: format!("{}-{}", pre, suf).into(),
                         };
                     }
                 }
@@ -120,6 +120,8 @@ pub struct TypeRegister {
     /// used to construct helpful error messages, such as "Row can only be within a GridLayout element".
     context_restricted_types: HashMap<String, HashSet<String>>,
     parent_registry: Option<Rc<RefCell<TypeRegister>>>,
+    /// If the lookup function should return types that are marked as internal
+    pub(crate) expose_internal_types: bool,
 }
 
 impl TypeRegister {
@@ -162,11 +164,11 @@ impl TypeRegister {
 
         declare_enum("TextHorizontalAlignment", &["left", "center", "right"]);
         declare_enum("TextVerticalAlignment", &["top", "center", "bottom"]);
-        declare_enum("TextWrap", &["no_wrap", "word_wrap"]);
+        declare_enum("TextWrap", &["no-wrap", "word-wrap"]);
         declare_enum("TextOverflow", &["clip", "elide"]);
         declare_enum(
             "LayoutAlignment",
-            &["stretch", "center", "start", "end", "space_between", "space_around"],
+            &["stretch", "center", "start", "end", "space-between", "space-around"],
         );
         declare_enum("ImageFit", &["fill", "contain", "cover"]);
         declare_enum("EventResult", &["reject", "accept"]);
@@ -206,7 +208,11 @@ impl TypeRegister {
     }
 
     pub fn new(parent: &Rc<RefCell<TypeRegister>>) -> Self {
-        Self { parent_registry: Some(parent.clone()), ..Default::default() }
+        Self {
+            parent_registry: Some(parent.clone()),
+            expose_internal_types: parent.borrow().expose_internal_types,
+            ..Default::default()
+        }
     }
 
     pub fn lookup(&self, name: &str) -> Type {
